@@ -118,10 +118,19 @@ class Command
          return flag_set("verbose");
          }
 
+      std::string get_passphrase(const std::string& prompt);
+
       bool flag_set(const std::string& flag_name) const;
 
       std::string get_arg(const std::string& opt_name) const;
 
+      /**
+      * Like get_arg but if the value is '-' then reads a passphrase from
+      * the terminal with echo suppressed.
+      */
+      std::string get_passphrase_arg(const std::string& prompt,
+                                     const std::string& opt_name);
+
       /*
       * Like get_arg() but if the argument was not specified or is empty, returns otherwise
       */
