@@ -11,6 +11,7 @@
 
 #include <botan/base64.h>
 #include <botan/hex.h>
+#include <botan/rng.h>
 
 #include <botan/pk_keys.h>
 #include <botan/x509_key.h>
@@ -18,6 +19,7 @@
 #include <botan/pkcs8.h>
 #include <botan/pubkey.h>
 #include <botan/workfactor.h>
+#include <botan/data_src.h>
 
 #if defined(BOTAN_HAS_DL_GROUP)
    #include <botan/dl_group.h>
@@ -32,7 +34,7 @@ namespace Botan_CLI {
 class PK_Keygen final : public Command
    {
    public:
-      PK_Keygen() : Command("keygen --algo=RSA --params= --passphrase= --pbe= --pbe-millis=300 --der-out") {}
+      PK_Keygen() : Command("keygen --algo=RSA --params= --passphrase= --pbe= --pbe-millis=300 --provider= --der-out") {}
 
       std::string group() const override
          {
@@ -48,16 +50,17 @@ class PK_Keygen final : public Command
          {
          const std::string algo = get_arg("algo");
          const std::string params = get_arg("params");
+         const std::string provider = get_arg("provider");
 
-         std::unique_ptr<Botan::Private_Key>
-         key(Botan::create_private_key(algo, rng(), params));
+         std::unique_ptr<Botan::Private_Key> key =
+            Botan::create_private_key(algo, rng(), params, provider);
 
          if(!key)
             {
             throw CLI_Error_Unsupported("keygen", algo);
             }
 
-         const std::string pass = get_arg("passphrase");
+         const std::string pass = get_passphrase_arg("Key passphrase", "passphrase");
          const bool der_out = flag_set("der-out");
 
          const std::chrono::milliseconds pbe_millis(get_arg_sz("pbe-millis"));
@@ -115,7 +118,7 @@ std::string algo_default_emsa(const std::string& key)
 class PK_Fingerprint final : public Command
    {
    public:
-      PK_Fingerprint() : Command("fingerprint --algo=SHA-256 *keys") {}
+      PK_Fingerprint() : Command("fingerprint --no-fsname --algo=SHA-256 *keys") {}
 
       std::string group() const override
          {
@@ -130,12 +133,18 @@ class PK_Fingerprint final : public Command
       void go() override
          {
          const std::string hash_algo = get_arg("algo");
+         const bool no_fsname = flag_set("no-fsname");
 
          for(std::string key_file : get_arg_list("keys"))
             {
             std::unique_ptr<Botan::Public_Key> key(Botan::X509::load_key(key_file));
 
-            output() << key_file << ": " << key->fingerprint_public(hash_algo) << "\n";
+            const std::string fprint = key->fingerprint_public(hash_algo);
+
+            if(no_fsname)
+               output() << fprint << "\n";
+            else
+               output() << key_file << ": " << fprint << "\n";
             }
          }
    };
@@ -159,11 +168,11 @@ class PK_Sign final : public Command
 
       void go() override
          {
-         std::unique_ptr<Botan::Private_Key> key(
-            Botan::PKCS8::load_key(
-               get_arg("key"),
-               rng(),
-               get_arg("passphrase")));
+         const std::string key_file = get_arg("key");
+         const std::string passphrase = get_passphrase_arg("Passphrase for " + key_file, "passphrase");
+
+         Botan::DataSource_Stream input(key_file);
+         std::unique_ptr<Botan::Private_Key> key = Botan::PKCS8::load_key(input, passphrase);;
 
          if(!key)
             {
@@ -256,17 +265,20 @@ class PKCS8_Tool final : public Command
 
       void go() override
          {
-         std::unique_ptr<Botan::Private_Key> key;
-         std::string pass_in = get_arg("pass-in");
+         const std::string key_file = get_arg("key");
+         const std::string pass_in = get_passphrase_arg("Password for " + key_file, "pass-in");
 
-         if (pass_in.empty())
-         {
-            key.reset(Botan::PKCS8::load_key(get_arg("key"), rng()));
-         }
+         Botan::DataSource_Memory key_src(slurp_file(key_file));
+         std::unique_ptr<Botan::Private_Key> key;
+
+         if(pass_in.empty())
+            {
+            key.reset(Botan::PKCS8::load_key(key_src, rng()));
+            }
          else
-         {
-            key.reset(Botan::PKCS8::load_key(get_arg("key"), rng(), pass_in));
-         }
+            {
+            key.reset(Botan::PKCS8::load_key(key_src, rng(), pass_in));
+            }
 
          const std::chrono::milliseconds pbe_millis(get_arg_sz("pbe-millis"));
          const std::string pbe = get_arg("pbe");
@@ -285,7 +297,7 @@ class PKCS8_Tool final : public Command
             }
          else
             {
-            const std::string pass_out = get_arg("pass-out");
+            const std::string pass_out = get_passphrase_arg("Passphrase to encrypt key", "pass-out");
 
             if(der_out)
                {
@@ -331,7 +343,7 @@ class EC_Group_Info final : public Command
 
       std::string description() const override
          {
-         return "Print raw elliptic curve domain parameters of the standarized curve name";
+         return "Print raw elliptic curve domain parameters of the standardized curve name";
          }
 
       void go() override
@@ -347,6 +359,7 @@ class EC_Group_Info final : public Command
             output() << "P = " << std::hex << group.get_p() << "\n"
                      << "A = " << std::hex << group.get_a() << "\n"
                      << "B = " << std::hex << group.get_b() << "\n"
+                     << "N = " << std::hex << group.get_order() << "\n"
                      << "G = " << group.get_g_x() << "," << group.get_g_y() << "\n";
             }
 
@@ -371,7 +384,7 @@ class DL_Group_Info final : public Command
 
       std::string description() const override
          {
-         return "Print raw Diffie-Hellman parameters (p,g) of the standarized DH group name";
+         return "Print raw Diffie-Hellman parameters (p,g) of the standardized DH group name";
          }
 
       void go() override
@@ -447,27 +460,24 @@ class Gen_DL_Group final : public Command
          const size_t qbits = get_arg_sz("qbits");
 
          const std::string type = get_arg("type");
+         const std::string seed_str = get_arg("seed");
 
          if(type == "strong")
             {
+            if(seed_str.size() > 0)
+               throw CLI_Usage_Error("Seed only supported for DSA param gen");
             Botan::DL_Group grp(rng(), Botan::DL_Group::Strong, pbits);
             output() << grp.PEM_encode(Botan::DL_Group::ANSI_X9_42);
             }
          else if(type == "subgroup")
             {
+            if(seed_str.size() > 0)
+               throw CLI_Usage_Error("Seed only supported for DSA param gen");
             Botan::DL_Group grp(rng(), Botan::DL_Group::Prime_Subgroup, pbits, qbits);
             output() << grp.PEM_encode(Botan::DL_Group::ANSI_X9_42);
             }
          else if(type == "dsa")
             {
-            const std::string seed_str = get_arg("seed");
-            const std::vector<uint8_t> seed = Botan::hex_decode(seed_str);
-
-            if(seed.empty())
-               {
-               throw CLI_Usage_Error("Generating DSA parameter set requires providing seed");
-               }
-
             size_t dsa_qbits = qbits;
             if(dsa_qbits == 0)
                {
@@ -479,8 +489,18 @@ class Gen_DL_Group final : public Command
                   throw CLI_Usage_Error("Invalid DSA p/q sizes");
                }
 
-            Botan::DL_Group grp(rng(), seed, pbits, dsa_qbits);
-            output() << grp.PEM_encode(Botan::DL_Group::ANSI_X9_42);
+            if(seed_str.empty())
+               {
+               Botan::DL_Group grp(rng(), Botan::DL_Group::DSA_Kosherizer, pbits, dsa_qbits);
+               output() << grp.PEM_encode(Botan::DL_Group::ANSI_X9_42);
+               }
+            else
+               {
+               const std::vector<uint8_t> seed = Botan::hex_decode(seed_str);
+               Botan::DL_Group grp(rng(), seed, pbits, dsa_qbits);
+               output() << grp.PEM_encode(Botan::DL_Group::ANSI_X9_42);
+               }
+
             }
          else
             {
