@@ -34,6 +34,8 @@ class TLS_All_Policy final : public Botan::TLS::Policy
             "AES-128/CCM(8)",
             "Camellia-256/GCM",
             "Camellia-128/GCM",
+            "ARIA-256/GCM",
+            "ARIA-128/GCM",
             "AES-256",
             "AES-128",
             "Camellia-256",
@@ -52,6 +54,10 @@ class TLS_All_Policy final : public Botan::TLS::Policy
          {
          return { "ECDSA", "RSA", "DSA" };
          }
+
+      bool allow_tls10() const override { return true; }
+      bool allow_tls11() const override { return true; }
+      bool allow_tls12() const override { return true; }
    };
 
 class TLS_Ciphersuites final : public Command
@@ -110,10 +116,14 @@ class TLS_Ciphersuites final : public Command
             {
             policy.reset(new Botan::TLS::Policy);
             }
-         else if(policy_type == "suiteb")
+         else if(policy_type == "suiteb_128")
             {
             policy.reset(new Botan::TLS::NSA_Suite_B_128);
             }
+         else if(policy_type == "suiteb_192")
+            {
+            policy.reset(new Botan::TLS::NSA_Suite_B_192);
+            }
          else if(policy_type == "strict")
             {
             policy.reset(new Botan::TLS::Strict_Policy);
@@ -128,6 +138,12 @@ class TLS_Ciphersuites final : public Command
             policy.reset(new Botan::TLS::Text_Policy(policy_txt));
             }
 
+         if(policy->acceptable_protocol_version(version) == false)
+            {
+            error_output() << "Error: the policy specified does not allow the given TLS version\n";
+            return;
+            }
+
          for(uint16_t suite_id : policy->ciphersuite_list(version, with_srp))
             {
             const Botan::TLS::Ciphersuite suite(Botan::TLS::Ciphersuite::by_id(suite_id));
