@@ -58,16 +58,19 @@ class Print_Help final : public Command
                }
             }
 
-         const std::map<std::string, std::string> groups_description
-            { { "encryption", "Encryption" },
+         const std::map<std::string, std::string> groups_description {
+            { "encryption", "Encryption" },
             { "compression", "Compression" },
+            { "codec", "Encoders/Decoders" },
             { "hash", "Hash Functions" },
             { "hmac", "HMAC" },
+            { "info", "Informational" },
             { "numtheory", "Number Theory" },
             { "passhash", "Password Hashing" },
             { "psk", "PSK Database" },
             { "pubkey", "Public Key Cryptography" },
             { "tls", "TLS" },
+            { "tss", "Secret Sharing" },
             { "x509", "X.509" },
             { "misc", "Miscellaneous" }
          };
@@ -116,6 +119,48 @@ class Print_Help final : public Command
 
 BOTAN_REGISTER_COMMAND("help", Print_Help);
 
+class Has_Command final : public Command
+   {
+   public:
+      Has_Command() : Command("has_command cmd") {}
+
+      std::string group() const override
+         {
+         return "info";
+         }
+
+      std::string description() const override
+         {
+         return "Test if a command is available";
+         }
+
+      void go() override
+         {
+         const std::string cmd = get_arg("cmd");
+
+         bool exists = false;
+         for(auto registered_cmd : Command::registered_cmds())
+            {
+            if(cmd == registered_cmd)
+               {
+               exists = true;
+               break;
+               }
+            }
+
+         if(verbose())
+            {
+            output() << "Command '" << cmd << "' is "
+                     << (exists ? "": "not ") << "available\n";
+            }
+
+         if(exists == false)
+            this->set_return_code(1);
+         }
+   };
+
+BOTAN_REGISTER_COMMAND("has_command", Has_Command);
+
 class Config_Info final : public Command
    {
    public:
@@ -132,7 +177,7 @@ class Config_Info final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "info";
          }
 
       std::string description() const override
@@ -178,7 +223,7 @@ class Version_Info final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "info";
          }
 
       std::string description() const override
@@ -208,7 +253,7 @@ class Print_Cpuid final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "info";
          }
 
       std::string description() const override
@@ -229,7 +274,7 @@ BOTAN_REGISTER_COMMAND("cpuid", Print_Cpuid);
 class Hash final : public Command
    {
    public:
-      Hash() : Command("hash --algo=SHA-256 --buf-size=4096 *files") {}
+      Hash() : Command("hash --algo=SHA-256 --buf-size=4096 --no-fsname *files") {}
 
       std::string group() const override
          {
@@ -252,6 +297,7 @@ class Hash final : public Command
             }
 
          const size_t buf_size = get_arg_sz("buf-size");
+         const bool no_fsname = flag_set("no-fsname");
 
          std::vector<std::string> files = get_arg_list("files");
          if(files.empty())
@@ -265,7 +311,12 @@ class Hash final : public Command
                {
                auto update_hash = [&](const uint8_t b[], size_t l) { hash_fn->update(b, l); };
                read_file(fsname, update_hash, buf_size);
-               output() << Botan::hex_encode(hash_fn->final()) << " " << fsname << "\n";
+               const std::string digest = Botan::hex_encode(hash_fn->final());
+
+               if(no_fsname)
+                  output() << digest << "\n";
+               else
+                  output() << digest << " " << fsname << "\n";
                }
             catch(CLI_IO_Error& e)
                {
@@ -362,7 +413,7 @@ class Hex_Encode final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "codec";
          }
 
       std::string description() const override
@@ -386,7 +437,7 @@ class Hex_Decode final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "codec";
          }
 
       std::string description() const override
@@ -419,7 +470,7 @@ class Base64_Encode final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "codec";
          }
 
       std::string description() const override
@@ -446,7 +497,7 @@ class Base64_Decode final : public Command
 
       std::string group() const override
          {
-         return "misc";
+         return "codec";
          }
 
       std::string description() const override
@@ -489,7 +540,7 @@ class Generate_Bcrypt final : public Command
 
       void go() override
          {
-         const std::string password = get_arg("password");
+         const std::string password = get_passphrase_arg("Passphrase to hash", "password");
          const size_t wf = get_arg_sz("work-factor");
 
          if(wf < 4 || wf > 18)
@@ -523,7 +574,7 @@ class Check_Bcrypt final : public Command
 
       void go() override
          {
-         const std::string password = get_arg("password");
+         const std::string password = get_passphrase_arg("Password to check", "password");
          const std::string hash = get_arg("hash");
 
          if(hash.length() != 60)
@@ -534,6 +585,9 @@ class Check_Bcrypt final : public Command
          const bool ok = Botan::check_bcrypt(password, hash);
 
          output() << "Password is " << (ok ? "valid" : "NOT valid") << std::endl;
+
+         if(ok == false)
+            set_return_code(1);
          }
    };
 
@@ -546,7 +600,7 @@ BOTAN_REGISTER_COMMAND("check_bcrypt", Check_Bcrypt);
 class HMAC final : public Command
    {
    public:
-      HMAC() : Command("hmac --hash=SHA-256 --buf-size=4096 key *files") {}
+      HMAC() : Command("hmac --hash=SHA-256 --buf-size=4096 --no-fsname key *files") {}
 
       std::string group() const override
          {
@@ -560,6 +614,7 @@ class HMAC final : public Command
 
       void go() override
          {
+         const bool no_fsname = flag_set("no-fsname");
          const std::string hash_algo = get_arg("hash");
          std::unique_ptr<Botan::MessageAuthenticationCode> hmac =
             Botan::MessageAuthenticationCode::create("HMAC(" + hash_algo + ")");
@@ -581,7 +636,12 @@ class HMAC final : public Command
                {
                auto update_hmac = [&](const uint8_t b[], size_t l) { hmac->update(b, l); };
                read_file(fsname, update_hmac, buf_size);
-               output() << Botan::hex_encode(hmac->final()) << " " << fsname << "\n";
+               output() << Botan::hex_encode(hmac->final());
+
+               if(no_fsname == false)
+                  output() << " " << fsname;
+
+               output() << "\n";
                }
             catch(CLI_IO_Error& e)
                {
