@@ -45,23 +45,25 @@ class Sign_Cert final : public Command
       void go() override
          {
          Botan::X509_Certificate ca_cert(get_arg("ca_cert"));
-         std::unique_ptr<Botan::Private_Key> key;
-         const std::string pass = get_arg("ca-key-pass");
+
+         const std::string key_file = get_arg("ca_key");
+         const std::string pass = get_passphrase_arg("Password for " + key_file, "ca-key-pass");
          const std::string emsa = get_arg("emsa");
          const std::string hash = get_arg("hash");
 
+         std::unique_ptr<Botan::Private_Key> key;
          if(!pass.empty())
             {
-            key.reset(Botan::PKCS8::load_key(get_arg("ca_key"), rng(), pass));
+            key.reset(Botan::PKCS8::load_key(key_file, rng(), pass));
             }
          else
             {
-            key.reset(Botan::PKCS8::load_key(get_arg("ca_key"), rng()));
+            key.reset(Botan::PKCS8::load_key(key_file, rng()));
             }
 
          if(!key)
             {
-            throw CLI_Error("Failed to load key from " + get_arg("ca_key"));
+            throw CLI_Error("Failed to load key from " + key_file);
             }
 
          std::map<std::string, std::string> options;
@@ -91,7 +93,7 @@ BOTAN_REGISTER_COMMAND("sign_cert", Sign_Cert);
 class Cert_Info final : public Command
    {
    public:
-      Cert_Info() : Command("cert_info --fingerprint --ber file") {}
+      Cert_Info() : Command("cert_info --fingerprint file") {}
 
       std::string group() const override
          {
@@ -105,7 +107,11 @@ class Cert_Info final : public Command
 
       void go() override
          {
-         Botan::DataSource_Stream in(get_arg("file"), flag_set("ber"));
+         const std::string arg_file = get_arg("file");
+
+         std::vector<uint8_t> data = slurp_file(get_arg("file"));
+
+         Botan::DataSource_Memory in(data);
 
          while(!in.end_of_data())
             {
@@ -233,7 +239,7 @@ class Gen_Self_Signed final : public Command
    public:
       Gen_Self_Signed()
          : Command("gen_self_signed key CN --country= --dns= "
-                   "--organization= --email= --key-pass= --ca --hash=SHA-256 --emsa=") {}
+                   "--organization= --email= --days=365 --key-pass= --ca --hash=SHA-256 --emsa= --der") {}
 
       std::string group() const override
          {
@@ -247,20 +253,25 @@ class Gen_Self_Signed final : public Command
 
       void go() override
          {
-         std::unique_ptr<Botan::Private_Key> key(Botan::PKCS8::load_key(get_arg("key"), rng(), get_arg("key-pass")));
+         const std::string key_file = get_arg("key");
+         const std::string passphrase = get_passphrase_arg("Passphrase for " + key_file, "key-pass");
+         std::unique_ptr<Botan::Private_Key> key(Botan::PKCS8::load_key(key_file, rng(), passphrase));
 
          if(!key)
             {
             throw CLI_Error("Failed to load key from " + get_arg("key"));
             }
 
-         Botan::X509_Cert_Options opts;
+         const uint32_t lifetime = static_cast<uint32_t>(get_arg_sz("days") * 24 * 60 * 60);
+
+         Botan::X509_Cert_Options opts("", lifetime);
 
          opts.common_name  = get_arg("CN");
          opts.country      = get_arg("country");
          opts.organization = get_arg("organization");
          opts.email        = get_arg("email");
          opts.more_dns = Botan::split_on(get_arg("dns"), ',');
+         const bool der_format = flag_set("der");
 
          std::string emsa = get_arg("emsa");
 
@@ -274,7 +285,13 @@ class Gen_Self_Signed final : public Command
 
          Botan::X509_Certificate cert = Botan::X509::create_self_signed_cert(opts, *key, get_arg("hash"), rng());
 
-         output() << cert.PEM_encode();
+         if(der_format)
+            {
+            auto der = cert.BER_encode();
+            output().write(reinterpret_cast<const char*>(der.data()), der.size());
+            }
+         else
+            output() << cert.PEM_encode();
          }
    };
 
@@ -285,7 +302,7 @@ class Generate_PKCS10 final : public Command
    public:
       Generate_PKCS10()
          : Command("gen_pkcs10 key CN --country= --organization= "
-                   "--email= --dns= --key-pass= --hash=SHA-256 --emsa=") {}
+                   "--email= --dns= --ext-ku= --key-pass= --hash=SHA-256 --emsa=") {}
 
       std::string group() const override
          {
@@ -314,6 +331,11 @@ class Generate_PKCS10 final : public Command
          opts.email        = get_arg("email");
          opts.more_dns     = Botan::split_on(get_arg("dns"), ',');
 
+         for(std::string ext_ku : Botan::split_on(get_arg("ext-ku"), ','))
+            {
+            opts.add_ex_constraint(ext_ku);
+            }
+
          std::string emsa = get_arg("emsa");
 
          if(emsa.empty() == false)
