@@ -59,7 +59,13 @@ std::string GCM_Mode::provider() const
 
 size_t GCM_Mode::update_granularity() const
    {
-   return GCM_BS;
+   return GCM_BS * std::max<size_t>(2, BOTAN_BLOCK_CIPHER_PAR_MULT);
+   }
+
+bool GCM_Mode::valid_nonce_length(size_t len) const
+   {
+   // GCM does not support empty nonces
+   return (len > 0);
    }
 
 Key_Length_Specification GCM_Mode::key_spec() const
@@ -86,7 +92,8 @@ void GCM_Mode::set_associated_data(const uint8_t ad[], size_t ad_len)
 
 void GCM_Mode::start_msg(const uint8_t nonce[], size_t nonce_len)
    {
-   // any size is valid for GCM nonce
+   if(!valid_nonce_length(nonce_len))
+      throw Invalid_IV_Length(name(), nonce_len);
 
    secure_vector<uint8_t> y0(GCM_BS);
 
@@ -110,7 +117,7 @@ void GCM_Mode::start_msg(const uint8_t nonce[], size_t nonce_len)
 
 size_t GCM_Encryption::process(uint8_t buf[], size_t sz)
    {
-   BOTAN_ARG_CHECK(sz % update_granularity() == 0);
+   BOTAN_ARG_CHECK(sz % update_granularity() == 0, "Invalid buffer size");
    m_ctr->cipher(buf, buf, sz);
    m_ghash->update(buf, sz);
    return sz;
@@ -118,7 +125,7 @@ size_t GCM_Encryption::process(uint8_t buf[], size_t sz)
 
 void GCM_Encryption::finish(secure_vector<uint8_t>& buffer, size_t offset)
    {
-   BOTAN_ARG_CHECK(offset <= buffer.size());
+   BOTAN_ARG_CHECK(offset <= buffer.size(), "Invalid offset");
    const size_t sz = buffer.size() - offset;
    uint8_t* buf = buffer.data() + offset;
 
@@ -130,7 +137,7 @@ void GCM_Encryption::finish(secure_vector<uint8_t>& buffer, size_t offset)
 
 size_t GCM_Decryption::process(uint8_t buf[], size_t sz)
    {
-   BOTAN_ARG_CHECK(sz % update_granularity() == 0);
+   BOTAN_ARG_CHECK(sz % update_granularity() == 0, "Invalid buffer size");
    m_ghash->update(buf, sz);
    m_ctr->cipher(buf, buf, sz);
    return sz;
@@ -138,12 +145,12 @@ size_t GCM_Decryption::process(uint8_t buf[], size_t sz)
 
 void GCM_Decryption::finish(secure_vector<uint8_t>& buffer, size_t offset)
    {
-   BOTAN_ARG_CHECK(offset <= buffer.size());
+   BOTAN_ARG_CHECK(offset <= buffer.size(), "Invalid offset");
    const size_t sz = buffer.size() - offset;
    uint8_t* buf = buffer.data() + offset;
 
    if(sz < tag_size())
-      throw Exception("Insufficient input for GCM decryption, tag missing");
+      throw Decoding_Error("Insufficient input for GCM decryption, tag missing");
 
    const size_t remaining = sz - tag_size();
 
