@@ -9,6 +9,7 @@
 
 #include <botan/gost_3410.h>
 #include <botan/internal/pk_ops_impl.h>
+#include <botan/internal/point_mul.h>
 #include <botan/reducer.h>
 #include <botan/der_enc.h>
 #include <botan/ber_dec.h>
@@ -34,16 +35,19 @@ std::vector<uint8_t> GOST_3410_PublicKey::public_key_bits() const
       std::swap(bits[part_size+i], bits[2*part_size-1-i]);
       }
 
-   return DER_Encoder().encode(bits, OCTET_STRING).get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(bits, OCTET_STRING);
+   return output;
    }
 
 AlgorithmIdentifier GOST_3410_PublicKey::algorithm_identifier() const
    {
-   std::vector<uint8_t> params =
-      DER_Encoder().start_cons(SEQUENCE)
+   std::vector<uint8_t> params;
+
+   DER_Encoder(params)
+      .start_cons(SEQUENCE)
          .encode(domain().get_curve_oid())
-         .end_cons()
-      .get_contents_unlocked();
+      .end_cons();
 
    return AlgorithmIdentifier(get_oid(), params);
    }
@@ -104,6 +108,8 @@ class GOST_3410_Signature_Operation final : public PK_Ops::Signature_with_EMSA
          m_x(gost_3410.private_value())
          {}
 
+      size_t signature_length() const override { return 2*m_group.get_order_bytes(); }
+
       size_t max_input_bits() const override { return m_group.get_order_bits(); }
 
       secure_vector<uint8_t> raw_sign(const uint8_t msg[], size_t msg_len,
@@ -151,7 +157,7 @@ class GOST_3410_Verification_Operation final : public PK_Ops::Verification_with_
                                        const std::string& emsa) :
          PK_Ops::Verification_with_EMSA(emsa),
          m_group(gost.domain()),
-         m_public_point(gost.public_point())
+         m_gy_mul(m_group.get_base_point(), gost.public_point())
          {}
 
       size_t max_input_bits() const override { return m_group.get_order_bits(); }
@@ -162,7 +168,7 @@ class GOST_3410_Verification_Operation final : public PK_Ops::Verification_with_
                   const uint8_t sig[], size_t sig_len) override;
    private:
       const EC_Group m_group;
-      const PointGFp& m_public_point;
+      const PointGFp_Multi_Point_Precompute m_gy_mul;
    };
 
 bool GOST_3410_Verification_Operation::verify(const uint8_t msg[], size_t msg_len,
@@ -184,12 +190,12 @@ bool GOST_3410_Verification_Operation::verify(const uint8_t msg[], size_t msg_le
    if(e == 0)
       e = 1;
 
-   const BigInt v = inverse_mod(e, order);
+   const BigInt v = m_group.inverse_mod_order(e);
 
    const BigInt z1 = m_group.multiply_mod_order(s, v);
    const BigInt z2 = m_group.multiply_mod_order(-r, v);
 
-   const PointGFp R = m_group.point_multiply(z1, m_public_point, z2);
+   const PointGFp R = m_gy_mul.multi_exp(z1, z2);
 
    if(R.is_zero())
      return false;
