@@ -97,7 +97,7 @@ secure_vector<uint8_t> PKCS8_decode(
       }
    catch(Decoding_Error& e)
       {
-      throw Decoding_Error("PKCS #8 private key decoding failed: " + std::string(e.what()));
+      throw Decoding_Error("PKCS #8 private key decoding", e);
       }
 
    try
@@ -105,7 +105,7 @@ secure_vector<uint8_t> PKCS8_decode(
       if(is_encrypted)
          {
          if(OIDS::lookup(pbe_alg_id.get_oid()) != "PBE-PKCS5v20")
-            throw Exception("Unknown PBE type " + pbe_alg_id.get_oid().as_string());
+            throw PKCS8_Exception("Unknown PBE type " + pbe_alg_id.get_oid().as_string());
 #if defined(BOTAN_HAS_PKCS5_PBES2)
          key = pbes2_decrypt(key_data, get_passphrase(), pbe_alg_id.get_parameters());
 #else
@@ -126,7 +126,7 @@ secure_vector<uint8_t> PKCS8_decode(
       }
    catch(std::exception& e)
       {
-      throw Decoding_Error("PKCS #8 private key decoding failed: " + std::string(e.what()));
+      throw Decoding_Error("PKCS #8 private key decoding", e);
       }
    return key;
    }
@@ -167,9 +167,14 @@ choose_pbe_params(const std::string& pbe_algo, const std::string& key_algo)
       }
 
    SCAN_Name request(pbe_algo);
-   if(request.algo_name() != "PBE-PKCS5v20" || request.arg_count() != 2)
-      throw Exception("Unsupported PBE " + pbe_algo);
-   return std::make_pair(request.arg(1), request.arg(0));
+
+   if(request.arg_count() != 2 ||
+      (request.algo_name() != "PBE-PKCS5v20" && request.algo_name() != "PBES2"))
+      {
+      throw Invalid_Argument("Unsupported PBE " + pbe_algo);
+      }
+
+   return std::make_pair(request.arg(0), request.arg(1));
    }
 
 }
@@ -192,12 +197,14 @@ std::vector<uint8_t> BER_encode(const Private_Key& key,
       pbes2_encrypt_msec(PKCS8::BER_encode(key), pass, msec, nullptr,
                          pbe_params.first, pbe_params.second, rng);
 
-   return DER_Encoder()
-         .start_cons(SEQUENCE)
-            .encode(pbe_info.first)
-            .encode(pbe_info.second, OCTET_STRING)
-         .end_cons()
-      .get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder der(output);
+   der.start_cons(SEQUENCE)
+         .encode(pbe_info.first)
+         .encode(pbe_info.second, OCTET_STRING)
+      .end_cons();
+
+   return output;
 #else
    BOTAN_UNUSED(key, rng, pass, msec, pbe_algo);
    throw Encoding_Error("PKCS8::BER_encode cannot encrypt because PBES2 was disabled in build");
@@ -238,12 +245,15 @@ std::vector<uint8_t> BER_encode_encrypted_pbkdf_iter(const Private_Key& key,
                          pbkdf_hash.empty() ? "SHA-256" : pbkdf_hash,
                          rng);
 
-   return DER_Encoder()
-         .start_cons(SEQUENCE)
-            .encode(pbe_info.first)
-            .encode(pbe_info.second, OCTET_STRING)
-         .end_cons()
-      .get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder der(output);
+   der.start_cons(SEQUENCE)
+         .encode(pbe_info.first)
+         .encode(pbe_info.second, OCTET_STRING)
+      .end_cons();
+
+   return output;
+
 #else
    BOTAN_UNUSED(key, rng, pass, pbkdf_iterations, cipher, pbkdf_hash);
    throw Encoding_Error("PKCS8::BER_encode_encrypted_pbkdf_iter cannot encrypt because PBES2 disabled in build");
@@ -284,12 +294,14 @@ std::vector<uint8_t> BER_encode_encrypted_pbkdf_msec(const Private_Key& key,
                          pbkdf_hash.empty() ? "SHA-256" : pbkdf_hash,
                          rng);
 
-   return DER_Encoder()
-         .start_cons(SEQUENCE)
-            .encode(pbe_info.first)
-            .encode(pbe_info.second, OCTET_STRING)
-         .end_cons()
-      .get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
+      .start_cons(SEQUENCE)
+         .encode(pbe_info.first)
+         .encode(pbe_info.second, OCTET_STRING)
+      .end_cons();
+
+   return output;
 #else
    BOTAN_UNUSED(key, rng, pass, pbkdf_msec, pbkdf_iterations, cipher, pbkdf_hash);
    throw Encoding_Error("BER_encode_encrypted_pbkdf_msec cannot encrypt because PBES2 disabled in build");
