@@ -15,6 +15,7 @@
 #include <botan/ber_dec.h>
 #include <botan/pow_mod.h>
 #include <botan/monty.h>
+#include <botan/divide.h>
 #include <botan/internal/monty_exp.h>
 
 #if defined(BOTAN_HAS_OPENSSL)
@@ -45,12 +46,14 @@ AlgorithmIdentifier RSA_PublicKey::algorithm_identifier() const
 
 std::vector<uint8_t> RSA_PublicKey::public_key_bits() const
    {
-   return DER_Encoder()
-      .start_cons(SEQUENCE)
+   std::vector<uint8_t> output;
+   DER_Encoder der(output);
+   der.start_cons(SEQUENCE)
          .encode(m_n)
          .encode(m_e)
-      .end_cons()
-      .get_contents_unlocked();
+      .end_cons();
+
+   return output;
    }
 
 RSA_PublicKey::RSA_PublicKey(const AlgorithmIdentifier&,
@@ -123,8 +126,8 @@ RSA_PrivateKey::RSA_PrivateKey(const BigInt& prime1,
       m_d = inverse_mod(m_e, phi_n);
       }
 
-   m_d1 = m_d % (m_p - 1);
-   m_d2 = m_d % (m_q - 1);
+   m_d1 = ct_modulo(m_d, m_p - 1);
+   m_d2 = ct_modulo(m_d, m_q - 1);
    }
 
 /*
@@ -141,17 +144,22 @@ RSA_PrivateKey::RSA_PrivateKey(RandomNumberGenerator& rng,
 
    m_e = exp;
 
+   const size_t p_bits = (bits + 1) / 2;
+   const size_t q_bits = bits - p_bits;
+
    do
       {
-      m_p = random_prime(rng, (bits + 1) / 2, m_e);
-      m_q = random_prime(rng, bits - m_p.bits(), m_e);
+      m_p = generate_rsa_prime(rng, rng, p_bits, m_e);
+      m_q = generate_rsa_prime(rng, rng, q_bits, m_e);
       m_n = m_p * m_q;
       } while(m_n.bits() != bits);
 
+   // FIXME: lcm calls gcd which is not const time
    const BigInt phi_n = lcm(m_p - 1, m_q - 1);
+   // FIXME: this uses binary ext gcd because phi_n is even
    m_d = inverse_mod(m_e, phi_n);
-   m_d1 = m_d % (m_p - 1);
-   m_d2 = m_d % (m_q - 1);
+   m_d1 = ct_modulo(m_d, m_p - 1);
+   m_d2 = ct_modulo(m_d, m_q - 1);
    m_c = inverse_mod(m_q, m_p);
    }
 
@@ -166,7 +174,7 @@ bool RSA_PrivateKey::check_key(RandomNumberGenerator& rng, bool strong) const
    if(m_d < 2 || m_p < 3 || m_q < 3 || m_p*m_q != m_n)
       return false;
 
-   if(m_d1 != m_d % (m_p - 1) || m_d2 != m_d % (m_q - 1) || m_c != inverse_mod(m_q, m_p))
+   if(m_d1 != ct_modulo(m_d, m_p - 1) || m_d2 != ct_modulo(m_d, m_q - 1) || m_c != inverse_mod(m_q, m_p))
       return false;
 
    const size_t prob = (strong) ? 128 : 12;
@@ -176,7 +184,7 @@ bool RSA_PrivateKey::check_key(RandomNumberGenerator& rng, bool strong) const
 
    if(strong)
       {
-      if((m_e * m_d) % lcm(m_p - 1, m_q - 1) != 1)
+      if(ct_modulo(m_e * m_d, lcm(m_p - 1, m_q - 1)) != 1)
          return false;
 
       return KeyPair::signature_consistency_check(rng, *this, "EMSA4(SHA-256)");
@@ -195,6 +203,8 @@ class RSA_Private_Operation
    protected:
       size_t get_max_input_bits() const { return (m_mod_bits - 1); }
 
+      const size_t exp_blinding_bits = 64;
+
       explicit RSA_Private_Operation(const RSA_PrivateKey& rsa, RandomNumberGenerator& rng) :
          m_key(rsa),
          m_mod_p(m_key.get_p()),
@@ -206,8 +216,11 @@ class RSA_Private_Operation
                    rng,
                    [this](const BigInt& k) { return m_powermod_e_n(k); },
                    [this](const BigInt& k) { return inverse_mod(k, m_key.get_n()); }),
+         m_blinding_bits(64),
          m_mod_bytes(m_key.get_n().bytes()),
-         m_mod_bits(m_key.get_n().bits())
+         m_mod_bits(m_key.get_n().bits()),
+         m_max_d1_bits(m_key.get_p().bits() + m_blinding_bits),
+         m_max_d2_bits(m_key.get_q().bits() + m_blinding_bits)
          {
          }
 
@@ -221,34 +234,55 @@ class RSA_Private_Operation
 
       BigInt private_op(const BigInt& m) const
          {
+         /*
+         TODO
+           Consider using Montgomery reduction instead of Barrett, using
+           the "Smooth RSA-CRT" method. https://eprint.iacr.org/2007/039.pdf
+         */
+
          const size_t powm_window = 4;
-         const size_t exp_blinding_bits = 64;
 
-         const BigInt d1_mask(m_blinder.rng(), exp_blinding_bits);
-         const BigInt d2_mask(m_blinder.rng(), exp_blinding_bits);
+         const BigInt d1_mask(m_blinder.rng(), m_blinding_bits);
 
-         const BigInt masked_d1 = m_key.get_d1() + (d1_mask * (m_key.get_p() - 1));
-         const BigInt masked_d2 = m_key.get_d2() + (d2_mask * (m_key.get_q() - 1));
-
-#if defined(BOTAN_TARGET_OS_HAS_THREADS)
-         auto future_j1 = std::async(std::launch::async, [this, &m, &masked_d1, powm_window]() {
-               auto powm_d1_p = monty_precompute(m_monty_p, m, powm_window);
-               return monty_execute(*powm_d1_p, masked_d1);
-            });
-
-         auto powm_d2_q = monty_precompute(m_monty_q, m, powm_window);
-         BigInt j2 = monty_execute(*powm_d2_q, masked_d2);
-         BigInt j1 = future_j1.get();
-#else
-         auto powm_d1_p = monty_precompute(m_monty_p, m, powm_window);
-         auto powm_d2_q = monty_precompute(m_monty_q, m, powm_window);
-
-         BigInt j1 = monty_execute(*powm_d1_p, masked_d1);
-         BigInt j2 = monty_execute(*powm_d2_q, masked_d2);
+#if defined(BOTAN_TARGET_OS_HAS_THREADS) && !defined(BOTAN_HAS_VALGRIND)
+   #define BOTAN_RSA_USE_ASYNC
 #endif
 
-         j1 = m_mod_p.reduce(sub_mul(j1, j2, m_key.get_c()));
+#if defined(BOTAN_RSA_USE_ASYNC)
+         auto future_j1 = std::async(std::launch::async, [this, &m, &d1_mask, powm_window]() {
+#endif
+         const BigInt masked_d1 = m_key.get_d1() + (d1_mask * (m_key.get_p() - 1));
+         auto powm_d1_p = monty_precompute(m_monty_p, m_mod_p.reduce(m), powm_window);
+         BigInt j1 = monty_execute(*powm_d1_p, masked_d1, m_max_d1_bits);
 
+#if defined(BOTAN_RSA_USE_ASYNC)
+         return j1;
+         });
+#endif
+
+         const BigInt d2_mask(m_blinder.rng(), m_blinding_bits);
+         const BigInt masked_d2 = m_key.get_d2() + (d2_mask * (m_key.get_q() - 1));
+         auto powm_d2_q = monty_precompute(m_monty_q, m_mod_q.reduce(m), powm_window);
+         const BigInt j2 = monty_execute(*powm_d2_q, masked_d2, m_max_d2_bits);
+
+#if defined(BOTAN_RSA_USE_ASYNC)
+         BigInt j1 = future_j1.get();
+#endif
+
+         /*
+         * To recover the final value from the CRT representation (j1,j2)
+         * we use Garner's algorithm:
+         * c = q^-1 mod p (this is precomputed)
+         * h = c*(j1-j2) mod p
+         * m = j2 + h*q
+         *
+         * We must avoid leaking if j1 >= j2 or not, as doing so allows deriving
+         * information about the secret prime. Do this by first adding p to j1,
+         * which should ensure the subtraction of j2 does not underflow. But
+         * this may still underflow if p and q are imbalanced in size.
+         */
+
+         j1 = m_mod_p.multiply(m_mod_p.reduce((m_key.get_p() + j1) - j2), m_key.get_c());
          return mul_add(j1, m_key.get_q(), j2);
          }
 
@@ -262,8 +296,11 @@ class RSA_Private_Operation
 
       Fixed_Exponent_Power_Mod m_powermod_e_n;
       Blinder m_blinder;
-      size_t m_mod_bytes;
-      size_t m_mod_bits;
+      const size_t m_blinding_bits;
+      const size_t m_mod_bytes;
+      const size_t m_mod_bits;
+      const size_t m_max_d1_bits;
+      const size_t m_max_d2_bits;
    };
 
 class RSA_Signature_Operation final : public PK_Ops::Signature_with_EMSA,
@@ -273,6 +310,8 @@ class RSA_Signature_Operation final : public PK_Ops::Signature_with_EMSA,
 
       size_t max_input_bits() const override { return get_max_input_bits(); }
 
+      size_t signature_length() const override { return m_key.get_n().bytes(); }
+
       RSA_Signature_Operation(const RSA_PrivateKey& rsa, const std::string& emsa, RandomNumberGenerator& rng) :
          PK_Ops::Signature_with_EMSA(emsa),
          RSA_Private_Operation(rsa, rng)
@@ -301,6 +340,8 @@ class RSA_Decryption_Operation final : public PK_Ops::Decryption_with_EME,
          {
          }
 
+      size_t plaintext_length(size_t) const override { return m_mod_bytes; }
+
       secure_vector<uint8_t> raw_decrypt(const uint8_t msg[], size_t msg_len) override
          {
          const BigInt m(msg, msg_len);
@@ -346,7 +387,12 @@ class RSA_Public_Operation
          m_monty_n(std::make_shared<Montgomery_Params>(m_n))
          {}
 
-      size_t get_max_input_bits() const { return (m_n.bits() - 1); }
+      size_t get_max_input_bits() const
+         {
+         const size_t n_bits = m_n.bits();
+         BOTAN_ASSERT_NOMSG(n_bits >= 384);
+         return n_bits - 1;
+         }
 
    protected:
       BigInt public_op(const BigInt& m) const
@@ -356,7 +402,7 @@ class RSA_Public_Operation
 
          const size_t powm_window = 1;
 
-         auto powm_m_n = monty_precompute(m_monty_n, m, powm_window);
+         auto powm_m_n = monty_precompute(m_monty_n, m, powm_window, false);
          return monty_execute_vartime(*powm_m_n, m_e);
          }
 
@@ -378,6 +424,8 @@ class RSA_Encryption_Operation final : public PK_Ops::Encryption_with_EME,
          {
          }
 
+      size_t ciphertext_length(size_t) const override { return m_n.bytes(); }
+
       size_t max_raw_input_bits() const override { return get_max_input_bits(); }
 
       secure_vector<uint8_t> raw_encrypt(const uint8_t msg[], size_t msg_len,
