@@ -87,7 +87,7 @@ Certificate_Req::Certificate_Req(const std::vector<uint8_t>& buf,
       if(cert_type_name.empty()) // something we don't know
          continue;
 
-      m_cert_key_types.push_back(cert_type_name);
+      m_cert_key_types.emplace_back(cert_type_name);
       }
 
    if(version.supports_negotiable_signature_algorithms())
@@ -115,7 +115,7 @@ Certificate_Req::Certificate_Req(const std::vector<uint8_t>& buf,
       BER_Decoder decoder(name_bits.data(), name_bits.size());
       X509_DN name;
       decoder.decode(name);
-      m_names.push_back(name);
+      m_names.emplace_back(name);
       }
    }
 
