@@ -398,6 +398,11 @@ Handshake_State::choose_sig_format(const Private_Key& key,
 
       for(Signature_Scheme scheme : schemes)
          {
+         if(signature_scheme_is_known(scheme) == false)
+            {
+            continue;
+            }
+
          if(signature_algorithm_of_scheme(scheme) == sig_algo)
             {
             if(std::find(allowed.begin(), allowed.end(), scheme) != allowed.end())
@@ -429,7 +434,7 @@ Handshake_State::choose_sig_format(const Private_Key& key,
       {
       if(sig_algo == "RSA")
          {
-         const std::string padding = "EMSA3(Parallel(MD5,SHA-160))";
+         const std::string padding = "PKCS1v15(Parallel(MD5,SHA-160))";
          return std::make_pair(padding, IEEE_1363);
          }
       else if(sig_algo == "DSA" || sig_algo == "ECDSA")
@@ -451,7 +456,8 @@ bool supported_algos_include(
    {
    for(Signature_Scheme scheme : schemes)
       {
-      if(hash_function_of_scheme(scheme) == hash_type &&
+      if(signature_scheme_is_known(scheme) &&
+         hash_function_of_scheme(scheme) == hash_type &&
          signature_algorithm_of_scheme(scheme) == key_type)
          {
          return true;
@@ -489,7 +495,7 @@ Handshake_State::parse_sig_format(const Public_Key& key,
 
       if(key_type == "RSA")
          {
-         const std::string padding = "EMSA3(Parallel(MD5,SHA-160))";
+         const std::string padding = "PKCS1v15(Parallel(MD5,SHA-160))";
          return std::make_pair(padding, IEEE_1363);
          }
       else if(key_type == "DSA" || key_type == "ECDSA")
