@@ -26,6 +26,9 @@
 #elif defined(BOTAN_TARGET_SUPPORTS_NEON)
   #include <arm_neon.h>
   #define BOTAN_SIMD_USE_NEON
+
+#else
+  #include <botan/rotate.h>
 #endif
 
 namespace Botan {
@@ -47,10 +50,8 @@ class SIMD_4x32 final
       SIMD_4x32& operator=(const SIMD_4x32& other) = default;
       SIMD_4x32(const SIMD_4x32& other) = default;
 
-#if !defined(BOTAN_BUILD_COMPILER_IS_MSVC_2013)
       SIMD_4x32& operator=(SIMD_4x32&& other) = default;
       SIMD_4x32(SIMD_4x32&& other) = default;
-#endif
 
       /**
       * Zero initialize SIMD register with 4 32-bit elements
@@ -133,31 +134,15 @@ class SIMD_4x32 final
 #if defined(BOTAN_SIMD_USE_SSE2)
          return SIMD_4x32(_mm_loadu_si128(reinterpret_cast<const __m128i*>(in)));
 #elif defined(BOTAN_SIMD_USE_ALTIVEC)
-         const uint32_t* in_32 = static_cast<const uint32_t*>(in);
 
-         __vector unsigned int R0 = vec_ld(0, in_32);
-         __vector unsigned int R1 = vec_ld(12, in_32);
+         uint32_t R[4];
+         Botan::load_le(R, static_cast<const uint8_t*>(in), 4);
+         return SIMD_4x32(R);
 
-         __vector unsigned char perm = vec_lvsl(0, in_32);
-
-         if(CPUID::is_big_endian())
-            {
-            perm = vec_xor(perm, vec_splat_u8(3)); // bswap vector
-            }
-
-         R0 = vec_perm(R0, R1, perm);
-
-         return SIMD_4x32(R0);
 #elif defined(BOTAN_SIMD_USE_NEON)
 
-         uint32_t in32[4];
-         std::memcpy(in32, in, 16);
-         if(CPUID::is_big_endian())
-            {
-            bswap_4(in32);
-            }
-         return SIMD_4x32(vld1q_u32(in32));
-
+         SIMD_4x32 l(vld1q_u32(static_cast<const uint32_t*>(in)));
+         return CPUID::is_big_endian() ? l.bswap() : l;
 #else
          SIMD_4x32 out;
          Botan::load_le(out.m_scalar, static_cast<const uint8_t*>(in), 4);
@@ -176,28 +161,14 @@ class SIMD_4x32 final
 
 #elif defined(BOTAN_SIMD_USE_ALTIVEC)
 
-         const uint32_t* in_32 = static_cast<const uint32_t*>(in);
-         __vector unsigned int R0 = vec_ld(0, in_32);
-         __vector unsigned int R1 = vec_ld(12, in_32);
-         __vector unsigned char perm = vec_lvsl(0, in_32);
-
-         if(CPUID::is_little_endian())
-            {
-            perm = vec_xor(perm, vec_splat_u8(3)); // bswap vector
-            }
-
-         R0 = vec_perm(R0, R1, perm);
-         return SIMD_4x32(R0);
+         uint32_t R[4];
+         Botan::load_be(R, static_cast<const uint8_t*>(in), 4);
+         return SIMD_4x32(R);
 
 #elif defined(BOTAN_SIMD_USE_NEON)
 
-         uint32_t in32[4];
-         std::memcpy(in32, in, 16);
-         if(CPUID::is_little_endian())
-            {
-            bswap_4(in32);
-            }
-         return SIMD_4x32(vld1q_u32(in32));
+         SIMD_4x32 l(vld1q_u32(static_cast<const uint32_t*>(in)));
+         return CPUID::is_little_endian() ? l.bswap() : l;
 
 #else
          SIMD_4x32 out;
@@ -228,14 +199,11 @@ class SIMD_4x32 final
 
          if(CPUID::is_big_endian())
             {
-            SIMD_4x32 swap = bswap();
-            swap.store_be(out);
+            bswap().store_le(out);
             }
          else
             {
-            uint32_t out32[4] = { 0 };
-            vst1q_u32(out32, m_neon);
-            copy_out_le(out, 16, out32);
+            vst1q_u8(out, vreinterpretq_u8_u32(m_neon));
             }
 #else
          Botan::store_le(out, m_scalar[0], m_scalar[1], m_scalar[2], m_scalar[3]);
@@ -264,14 +232,11 @@ class SIMD_4x32 final
 
          if(CPUID::is_little_endian())
             {
-            SIMD_4x32 swap = bswap();
-            swap.store_le(out);
+            bswap().store_le(out);
             }
          else
             {
-            uint32_t out32[4] = { 0 };
-            vst1q_u32(out32, m_neon);
-            copy_out_be(out, 16, out32);
+            vst1q_u8(out, vreinterpretq_u8_u32(m_neon));
             }
 
 #else
@@ -287,55 +252,10 @@ class SIMD_4x32 final
       template<size_t ROT1, size_t ROT2, size_t ROT3>
       SIMD_4x32 rho() const
          {
-         SIMD_4x32 res;
-
-#if defined(BOTAN_SIMD_USE_SSE2)
-
-         res.m_sse = _mm_or_si128(_mm_slli_epi32(m_sse, static_cast<int>(32-ROT1)),
-                                  _mm_srli_epi32(m_sse, static_cast<int>(ROT1)));
-         res.m_sse = _mm_xor_si128(
-            res.m_sse,
-            _mm_or_si128(_mm_slli_epi32(m_sse, static_cast<int>(32-ROT2)),
-                         _mm_srli_epi32(m_sse, static_cast<int>(ROT2))));
-         res.m_sse = _mm_xor_si128(
-            res.m_sse,
-            _mm_or_si128(_mm_slli_epi32(m_sse, static_cast<int>(32-ROT3)),
-                         _mm_srli_epi32(m_sse, static_cast<int>(ROT3))));
-
-#elif defined(BOTAN_SIMD_USE_ALTIVEC)
-
-         const unsigned int r1 = static_cast<unsigned int>(32-ROT1);
-         const unsigned int r2 = static_cast<unsigned int>(32-ROT2);
-         const unsigned int r3 = static_cast<unsigned int>(32-ROT3);
-         res.m_vmx = vec_rl(m_vmx, (__vector unsigned int){r1, r1, r1, r1});
-         res.m_vmx = vec_xor(res.m_vmx, vec_rl(m_vmx, (__vector unsigned int){r2, r2, r2, r2}));
-         res.m_vmx = vec_xor(res.m_vmx, vec_rl(m_vmx, (__vector unsigned int){r3, r3, r3, r3}));
-
-#elif defined(BOTAN_SIMD_USE_NEON)
-         res.m_neon = vorrq_u32(vshlq_n_u32(m_neon, static_cast<int>(32-ROT1)),
-                                vshrq_n_u32(m_neon, static_cast<int>(ROT1)));
-
-         res.m_neon = veorq_u32(
-            res.m_neon,
-            vorrq_u32(vshlq_n_u32(m_neon, static_cast<int>(32-ROT2)),
-                      vshrq_n_u32(m_neon, static_cast<int>(ROT2))));
-
-         res.m_neon = veorq_u32(
-            res.m_neon,
-            vorrq_u32(vshlq_n_u32(m_neon, static_cast<int>(32-ROT3)),
-                      vshrq_n_u32(m_neon, static_cast<int>(ROT3))));
-
-#else
-
-         for(size_t i = 0; i != 4; ++i)
-            {
-            res.m_scalar[i] = Botan::rotr<ROT1>(m_scalar[i]) ^
-                              Botan::rotr<ROT2>(m_scalar[i]) ^
-                              Botan::rotr<ROT3>(m_scalar[i]);
-            }
-#endif
-
-         return res;
+         const SIMD_4x32 rot1 = this->rotr<ROT1>();
+         const SIMD_4x32 rot2 = this->rotr<ROT2>();
+         const SIMD_4x32 rot3 = this->rotr<ROT3>();
+         return (rot1 ^ rot2 ^ rot3);
          }
 
       /**
@@ -357,9 +277,32 @@ class SIMD_4x32 final
          return SIMD_4x32(vec_rl(m_vmx, (__vector unsigned int){r, r, r, r}));
 
 #elif defined(BOTAN_SIMD_USE_NEON)
+
+         #if defined(BOTAN_TARGET_ARCH_IS_ARM32)
+
          return SIMD_4x32(vorrq_u32(vshlq_n_u32(m_neon, static_cast<int>(ROT)),
                                     vshrq_n_u32(m_neon, static_cast<int>(32-ROT))));
 
+         #else
+
+         if(ROT == 8)
+            {
+            const uint8_t maskb[16] = { 3,0,1,2, 7,4,5,6, 11,8,9,10, 15,12,13,14 };
+            const uint8x16_t mask = vld1q_u8(maskb);
+            return SIMD_4x32(vreinterpretq_u32_u8(vqtbl1q_u8(vreinterpretq_u8_u32(m_neon), mask)));
+            }
+         else if(ROT == 16)
+            {
+            return SIMD_4x32(vreinterpretq_u32_u16(vrev32q_u16(vreinterpretq_u16_u32(m_neon))));
+            }
+         else
+            {
+            return SIMD_4x32(vorrq_u32(vshlq_n_u32(m_neon, static_cast<int>(ROT)),
+                                       vshrq_n_u32(m_neon, static_cast<int>(32-ROT))));
+            }
+
+         #endif
+
 #else
          return SIMD_4x32(Botan::rotl<ROT>(m_scalar[0]),
                           Botan::rotl<ROT>(m_scalar[1]),
@@ -593,21 +536,19 @@ class SIMD_4x32 final
 
 #elif defined(BOTAN_SIMD_USE_ALTIVEC)
 
-         __vector unsigned char perm = vec_lvsl(0, static_cast<uint32_t*>(nullptr));
-         perm = vec_xor(perm, vec_splat_u8(3));
-         return SIMD_4x32(vec_perm(m_vmx, m_vmx, perm));
+         union {
+            __vector unsigned int V;
+            uint32_t R[4];
+            } vec;
+
+         vec.V = m_vmx;
+         bswap_4(vec.R);
+         return SIMD_4x32(vec.R[0], vec.R[1], vec.R[2], vec.R[3]);
 
 #elif defined(BOTAN_SIMD_USE_NEON)
 
-         //return SIMD_4x32(vrev64q_u32(m_neon));
+         return SIMD_4x32(vreinterpretq_u32_u8(vrev32q_u8(vreinterpretq_u8_u32(m_neon))));
 
-         // FIXME this is really slow
-         SIMD_4x32 ror8 = this->rotr<8>();
-         SIMD_4x32 rol8 = this->rotl<8>();
-
-         const SIMD_4x32 mask1 = SIMD_4x32::splat(0xFF00FF00);
-         const SIMD_4x32 mask2 = SIMD_4x32::splat(0x00FF00FF);
-         return (ror8 & mask1) | (rol8 & mask2);
 #else
          // scalar
          return SIMD_4x32(reverse_bytes(m_scalar[0]),
@@ -706,8 +647,6 @@ class SIMD_4x32 final
 #endif
    };
 
-typedef SIMD_4x32 SIMD_32;
-
 }
 
 #endif
