@@ -99,9 +99,10 @@ Extensions::create_extn_obj(const OID& oid,
       {
       extn->decode_inner(body);
       }
-   catch(Decoding_Error& e)
+   catch(Decoding_Error&)
       {
-      throw Decoding_Error("Decoding X.509 extension " + oid.as_string() + " failed", e.what());
+      extn.reset(new Cert_Extension::Unknown_Extension(oid, critical));
+      extn->decode_inner(body);
       }
    return extn;
    }
@@ -300,15 +301,16 @@ size_t Basic_Constraints::get_path_limit() const
 */
 std::vector<uint8_t> Basic_Constraints::encode_inner() const
    {
-   return DER_Encoder()
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
       .start_cons(SEQUENCE)
       .encode_if(m_is_ca,
                  DER_Encoder()
                     .encode(m_is_ca)
                     .encode_optional(m_path_limit, NO_CERT_PATH_LIMIT)
          )
-      .end_cons()
-   .get_contents_unlocked();
+      .end_cons();
+   return output;
    }
 
 /*
@@ -343,7 +345,7 @@ std::vector<uint8_t> Key_Usage::encode_inner() const
    if(m_constraints == NO_CONSTRAINTS)
       throw Encoding_Error("Cannot encode zero usage constraints");
 
-   const size_t unused_bits = low_bit(m_constraints) - 1;
+   const size_t unused_bits = ctz(static_cast<uint32_t>(m_constraints));
 
    std::vector<uint8_t> der;
    der.push_back(BIT_STRING);
@@ -404,7 +406,9 @@ void Key_Usage::contents_to(Data_Store& subject, Data_Store&) const
 */
 std::vector<uint8_t> Subject_Key_ID::encode_inner() const
    {
-   return DER_Encoder().encode(m_key_id, OCTET_STRING).get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(m_key_id, OCTET_STRING);
+   return output;
    }
 
 /*
@@ -434,6 +438,11 @@ Subject_Key_ID::Subject_Key_ID(const std::vector<uint8_t>& pub_key, const std::s
 
    hash->update(pub_key);
    hash->final(m_key_id.data());
+
+   // Truncate longer hashes, 192 bits here seems plenty
+   const size_t max_skid_len = (192 / 8);
+   if(m_key_id.size() > max_skid_len)
+      m_key_id.resize(max_skid_len);
    }
 
 /*
@@ -441,11 +450,12 @@ Subject_Key_ID::Subject_Key_ID(const std::vector<uint8_t>& pub_key, const std::s
 */
 std::vector<uint8_t> Authority_Key_ID::encode_inner() const
    {
-   return DER_Encoder()
-         .start_cons(SEQUENCE)
-            .encode(m_key_id, OCTET_STRING, ASN1_Tag(0), CONTEXT_SPECIFIC)
-         .end_cons()
-      .get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
+      .start_cons(SEQUENCE)
+         .encode(m_key_id, OCTET_STRING, ASN1_Tag(0), CONTEXT_SPECIFIC)
+      .end_cons();
+   return output;
    }
 
 /*
@@ -472,7 +482,9 @@ void Authority_Key_ID::contents_to(Data_Store&, Data_Store& issuer) const
 */
 std::vector<uint8_t> Subject_Alternative_Name::encode_inner() const
    {
-   return DER_Encoder().encode(m_alt_name).get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(m_alt_name);
+   return output;
    }
 
 /*
@@ -480,7 +492,9 @@ std::vector<uint8_t> Subject_Alternative_Name::encode_inner() const
 */
 std::vector<uint8_t> Issuer_Alternative_Name::encode_inner() const
    {
-   return DER_Encoder().encode(m_alt_name).get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(m_alt_name);
+   return output;
    }
 
 /*
@@ -521,11 +535,12 @@ void Issuer_Alternative_Name::contents_to(Data_Store&, Data_Store& issuer_info)
 */
 std::vector<uint8_t> Extended_Key_Usage::encode_inner() const
    {
-   return DER_Encoder()
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
       .start_cons(SEQUENCE)
          .encode_list(m_oids)
-      .end_cons()
-   .get_contents_unlocked();
+      .end_cons();
+   return output;
    }
 
 /*
@@ -719,11 +734,12 @@ std::vector<uint8_t> Certificate_Policies::encode_inner() const
    for(size_t i = 0; i != m_oids.size(); ++i)
       policies.push_back(Policy_Information(m_oids[i]));
 
-   return DER_Encoder()
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
       .start_cons(SEQUENCE)
          .encode_list(policies)
-      .end_cons()
-   .get_contents_unlocked();
+      .end_cons();
+   return output;
    }
 
 /*
@@ -766,13 +782,15 @@ std::vector<uint8_t> Authority_Information_Access::encode_inner() const
    {
    ASN1_String url(m_ocsp_responder, IA5_STRING);
 
-   return DER_Encoder()
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
       .start_cons(SEQUENCE)
       .start_cons(SEQUENCE)
       .encode(OIDS::lookup("PKIX.OCSP"))
       .add_object(ASN1_Tag(6), CONTEXT_SPECIFIC, url.value())
       .end_cons()
-      .end_cons().get_contents_unlocked();
+      .end_cons();
+   return output;
    }
 
 void Authority_Information_Access::decode_inner(const std::vector<uint8_t>& in)
@@ -842,7 +860,9 @@ CRL_Number* CRL_Number::copy() const
 */
 std::vector<uint8_t> CRL_Number::encode_inner() const
    {
-   return DER_Encoder().encode(m_crl_number).get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(m_crl_number);
+   return output;
    }
 
 /*
@@ -867,9 +887,9 @@ void CRL_Number::contents_to(Data_Store& info, Data_Store&) const
 */
 std::vector<uint8_t> CRL_ReasonCode::encode_inner() const
    {
-   return DER_Encoder()
-      .encode(static_cast<size_t>(m_reason), ENUMERATED, UNIVERSAL)
-   .get_contents_unlocked();
+   std::vector<uint8_t> output;
+   DER_Encoder(output).encode(static_cast<size_t>(m_reason), ENUMERATED, UNIVERSAL);
+   return output;
    }
 
 /*
