@@ -29,14 +29,14 @@ struct Pss_params
 
 Pss_params decode_pss_params(const std::vector<uint8_t>& encoded_pss_params)
    {
+   const AlgorithmIdentifier default_hash("SHA-160", AlgorithmIdentifier::USE_NULL_PARAM);
+   const AlgorithmIdentifier default_mgf("MGF1", default_hash.BER_encode());
+
    Pss_params pss_parameter;
    BER_Decoder(encoded_pss_params)
       .start_cons(SEQUENCE)
-         .decode_optional(pss_parameter.hash_algo, ASN1_Tag(0), PRIVATE, AlgorithmIdentifier("SHA-160",
-                    AlgorithmIdentifier::USE_NULL_PARAM))
-         .decode_optional(pss_parameter.mask_gen_algo, ASN1_Tag(1), PRIVATE,
-                    AlgorithmIdentifier("MGF1", DER_Encoder().encode(AlgorithmIdentifier("SHA-160",
-                                        AlgorithmIdentifier::USE_NULL_PARAM)).get_contents_unlocked()))
+         .decode_optional(pss_parameter.hash_algo, ASN1_Tag(0), PRIVATE, default_hash)
+         .decode_optional(pss_parameter.mask_gen_algo, ASN1_Tag(1), PRIVATE, default_mgf)
          .decode_optional(pss_parameter.salt_len, ASN1_Tag(2), PRIVATE, size_t(20))
          .decode_optional(pss_parameter.trailer_field, ASN1_Tag(3), PRIVATE, size_t(1))
       .end_cons();
@@ -85,7 +85,7 @@ void X509_Object::load_data(DataSource& in)
       }
    catch(Decoding_Error& e)
       {
-      throw Decoding_Error(PEM_label() + " decoding failed: " + e.what());
+      throw Decoding_Error(PEM_label() + " decoding", e);
       }
    }
 
@@ -117,16 +117,6 @@ void X509_Object::decode_from(BER_Decoder& from)
    force_decode();
    }
 
-/*
-* Return a BER encoded X.509 object
-*/
-std::vector<uint8_t> X509_Object::BER_encode() const
-   {
-   DER_Encoder der;
-   encode_into(der);
-   return der.get_contents_unlocked();
-   }
-
 /*
 * Return a PEM encoded X.509 object
 */
@@ -180,7 +170,7 @@ std::string X509_Object::hash_used_for_signature() const
 bool X509_Object::check_signature(const Public_Key* pub_key) const
    {
    if(!pub_key)
-      throw Exception("No key provided for " + PEM_label() + " signature check");
+      throw Invalid_Argument("No key provided for " + PEM_label() + " signature check");
    std::unique_ptr<const Public_Key> key(pub_key);
    return check_signature(*key);
    }
@@ -284,13 +274,15 @@ std::vector<uint8_t> X509_Object::make_signed(PK_Signer* signer,
    {
    const std::vector<uint8_t> signature = signer->sign_message(tbs_bits, rng);
 
-   return DER_Encoder()
+   std::vector<uint8_t> output;
+   DER_Encoder(output)
       .start_cons(SEQUENCE)
          .raw_bytes(tbs_bits)
          .encode(algo)
          .encode(signature, BIT_STRING)
-      .end_cons()
-   .get_contents_unlocked();
+      .end_cons();
+
+   return output;
    }
 
 namespace {
