@@ -92,7 +92,8 @@ PKIX::check_chain(const std::vector<std::shared_ptr<const X509_Certificate>>& ce
          }
 
       // Check the subject's DN components' length
-      for(const auto& dn_pair : subject->subject_dn().get_attributes())
+
+      for(const auto& dn_pair : subject->subject_dn().dn_info())
          {
          const size_t dn_ub = X509_DN::lookup_ub(dn_pair.first);
          // dn_pair = <OID,str>
@@ -371,26 +372,34 @@ PKIX::check_ocsp_online(const std::vector<std::shared_ptr<const X509_Certificate
       if(subject->ocsp_responder() == "")
          {
          ocsp_response_futures.emplace_back(std::async(std::launch::deferred, [&]() -> std::shared_ptr<const OCSP::Response> {
-                  throw Exception("No OCSP responder URL set for this certificate");
+                  return std::make_shared<const OCSP::Response>(Certificate_Status_Code::OSCP_NO_REVOCATION_URL);
                   }));
-            }
-         else
-            {
-            ocsp_response_futures.emplace_back(std::async(std::launch::async, [&]() -> std::shared_ptr<const OCSP::Response> {
-                  OCSP::Request req(*issuer, BigInt::decode(subject->serial_number()));
+         }
+      else
+         {
+         ocsp_response_futures.emplace_back(std::async(std::launch::async, [&]() -> std::shared_ptr<const OCSP::Response> {
+               OCSP::Request req(*issuer, BigInt::decode(subject->serial_number()));
 
-                  auto http = HTTP::POST_sync(subject->ocsp_responder(),
-                                              "application/ocsp-request",
-                                              req.BER_encode(),
-                                              /*redirects*/1,
-                                              timeout);
+               HTTP::Response http;
+               try
+                  {
+                  http = HTTP::POST_sync(subject->ocsp_responder(),
+                                                "application/ocsp-request",
+                                                req.BER_encode(),
+                                                /*redirects*/1,
+                                                timeout);
+                  }
+               catch(std::exception&)
+                  {
+                  // log e.what() ?
+                  }
+               if (http.status_code() != 200)
+                  return std::make_shared<const OCSP::Response>(Certificate_Status_Code::OSCP_SERVER_NOT_AVAILABLE);
+               // Check the MIME type?
 
-                  http.throw_unless_ok();
-                  // Check the MIME type?
-
-                  return std::make_shared<const OCSP::Response>(http.body());
-                  }));
-            }
+               return std::make_shared<const OCSP::Response>(http.body());
+               }));
+         }
       }
 
    std::vector<std::shared_ptr<const OCSP::Response>> ocsp_responses;
@@ -420,9 +429,10 @@ PKIX::check_crl_online(const std::vector<std::shared_ptr<const X509_Certificate>
 
    for(size_t i = 0; i != cert_path.size(); ++i)
       {
-      for(size_t c = 0; c != certstores.size(); ++i)
+      const std::shared_ptr<const X509_Certificate>& cert = cert_path.at(i);
+      for(size_t c = 0; c != certstores.size(); ++c)
          {
-         crls[i] = certstores[i]->find_crl_for(*cert_path[i]);
+         crls[i] = certstores[c]->find_crl_for(*cert);
          if(crls[i])
             break;
          }
@@ -438,17 +448,19 @@ PKIX::check_crl_online(const std::vector<std::shared_ptr<const X509_Certificate>
          */
          future_crls.emplace_back(std::future<std::shared_ptr<const X509_CRL>>());
          }
-      else if(cert_path[i]->crl_distribution_point() == "")
+      else if(cert->crl_distribution_point() == "")
          {
          // Avoid creating a thread for this case
          future_crls.emplace_back(std::async(std::launch::deferred, [&]() -> std::shared_ptr<const X509_CRL> {
-               throw Exception("No CRL distribution point for this certificate");
+               throw Not_Implemented("No CRL distribution point for this certificate");
                }));
          }
       else
          {
          future_crls.emplace_back(std::async(std::launch::async, [&]() -> std::shared_ptr<const X509_CRL> {
-               auto http = HTTP::GET_sync(cert_path[i]->crl_distribution_point());
+               auto http = HTTP::GET_sync(cert->crl_distribution_point(),
+                                          /*redirects*/ 1, timeout);
+
                http.throw_unless_ok();
                // check the mime type?
                return std::make_shared<const X509_CRL>(http.body());
@@ -462,16 +474,12 @@ PKIX::check_crl_online(const std::vector<std::shared_ptr<const X509_Certificate>
          {
          try
             {
-            std::future_status status = future_crls[i].wait_for(timeout);
-
-            if(status == std::future_status::ready)
-               {
-               crls[i] = future_crls[i].get();
-               }
+            crls[i] = future_crls[i].get();
             }
          catch(std::exception&)
             {
             // crls[i] left null
+            // todo: log exception e.what() ?
             }
          }
       }
@@ -733,7 +741,7 @@ PKIX::build_all_certificate_paths(std::vector<std::vector<std::shared_ptr<const
    if(cert_paths_out.empty())
       {
       if(stats.empty())
-         throw Exception("X509 path building failed for unknown reasons");
+         throw Internal_Error("X509 path building failed for unknown reasons");
       else
          // arbitrarily return the first error
          return stats[0];
@@ -774,7 +782,9 @@ void PKIX::merge_revocation_status(CertificatePathStatusCodes& chain_status,
          {
          for(auto&& code : ocsp[i])
             {
-            if(code == Certificate_Status_Code::OCSP_RESPONSE_GOOD)
+            if(code == Certificate_Status_Code::OCSP_RESPONSE_GOOD ||
+               code == Certificate_Status_Code::OSCP_NO_REVOCATION_URL ||  // softfail
+               code == Certificate_Status_Code::OSCP_SERVER_NOT_AVAILABLE) // softfail
                {
                had_ocsp = true;
                }
@@ -995,9 +1005,9 @@ Path_Validation_Result::Path_Validation_Result(CertificatePathStatusCodes status
 const X509_Certificate& Path_Validation_Result::trust_root() const
    {
    if(m_cert_path.empty())
-      throw Exception("Path_Validation_Result::trust_root no path set");
+      throw Invalid_State("Path_Validation_Result::trust_root no path set");
    if(result() != Certificate_Status_Code::VERIFIED)
-      throw Exception("Path_Validation_Result::trust_root meaningless with invalid status");
+      throw Invalid_State("Path_Validation_Result::trust_root meaningless with invalid status");
 
    return *m_cert_path[m_cert_path.size()-1];
    }
@@ -1019,7 +1029,10 @@ bool Path_Validation_Result::successful_validation() const
 
 bool Path_Validation_Result::no_warnings() const
    {
-   return m_warnings.empty();
+   for(auto status_set_i : m_warnings) 
+      if(!status_set_i.empty())
+         return false;
+   return true;
    }
 
 CertificatePathStatusCodes Path_Validation_Result::warnings() const
@@ -1040,4 +1053,18 @@ const char* Path_Validation_Result::status_string(Certificate_Status_Code code)
    return "Unknown error";
    }
 
+std::string Path_Validation_Result::warnings_string() const
+   {
+   const std::string sep(", ");
+   std::string res;
+   for(size_t i = 0; i < m_warnings.size(); i++)
+      {
+      for(auto code : m_warnings[i])
+         res += "[" + std::to_string(i) + "] " + status_string(code) + sep;
+      }
+   // remove last sep
+   if(res.size() >= sep.size())
+      res = res.substr(0, res.size() - sep.size());
+   return res;
+   }
 }
