@@ -41,17 +41,18 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
     if is_cross_target:
         if target_os == 'osx':
             target_os = 'ios'
-        elif target == 'cross-win32':
+        elif target == 'cross-win64':
             target_os = 'mingw'
 
     make_prefix = []
     test_prefix = []
     test_cmd = [os.path.join(root_dir, 'botan-test')]
 
-    fast_tests = ['block', 'aead', 'hash', 'stream', 'mac', 'modes',
+    fast_tests = ['block', 'aead', 'hash', 'stream', 'mac', 'modes', 'kdf',
                   'hmac_drbg', 'hmac_drbg_unit',
                   'tls', 'ffi',
-                  'rsa_sign', 'rsa_verify', 'dh_kat', 'ecdsa_sign', 'curve25519_scalar',
+                  'rsa_sign', 'rsa_verify', 'dh_kat',
+                  'ecc_randomized', 'ecdh_kat', 'ecdsa_sign', 'curve25519_scalar',
                   'simd_32', 'os_utils', 'util', 'util_dates']
 
     install_prefix = os.path.join(tempfile.gettempdir(), 'botan-install')
@@ -59,7 +60,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
              '--cc=%s' % (target_cc),
              '--os=%s' % (target_os)]
 
-    if target_cpu != None:
+    if target_cpu is not None:
         flags += ['--cpu=%s' % (target_cpu)]
 
     if target in ['shared', 'mini-shared']:
@@ -77,11 +78,14 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         # much faster compiling via the amalgamation than individual files.
         flags += ['--amalgamation']
 
+        if target_cc == 'msvc':
+            flags += ['--single-amalgamation-file']
+
     if target in ['bsi', 'nist']:
         # Arbitrarily test disable static on module policy builds
-        # x509 is optional for bsi/nist but add it so verify tests work with these minimized configs
+        # tls is optional for bsi/nist but add it so verify tests work with these minimized configs
         flags += ['--module-policy=%s' % (target),
-                  '--enable-modules=x509',
+                  '--enable-modules=tls',
                   '--disable-static']
 
     if target == 'docs':
@@ -89,9 +93,10 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         test_cmd = None
 
     if target == 'coverage':
-        flags += ['--with-coverage-info']
+        flags += ['--with-coverage-info', '--test-mode']
     if target == 'valgrind':
-        flags += ['--with-valgrind']
+        # valgrind in 16.04 has a bug with rdrand handling
+        flags += ['--with-valgrind', '--disable-rdrand']
         test_prefix = ['valgrind', '--error-exitcode=9', '-v', '--leak-check=full', '--show-reachable=yes']
         test_cmd += fast_tests
     if target == 'fuzzers':
@@ -108,6 +113,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
             flags += ['--with-debug-info']
         else:
             flags += ['--with-sanitizers']
+        flags += ['--with-debug-asserts']
+
     if target in ['valgrind', 'sanitizer', 'fuzzers']:
         flags += ['--disable-modules=locking_allocator']
 
@@ -118,8 +125,13 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         if target_os != 'linux' or target_cc != 'clang':
             raise Exception('Only Linux/clang supported in Sonar target currently')
 
+        # Use of -Os here is a little odd, but the issue is that the build time
+        # is quite long (because ccache from Xenial doesn't work right with
+        # these profiling flags) but we can't use --no-optimizations as that
+        # will make running the tests too slow.
         flags += ['--cc-abi-flags=-fprofile-instr-generate -fcoverage-mapping',
-                  '--disable-shared']
+                  '--disable-shared',
+                  '--optimize-for-size']
 
         make_prefix = [os.path.join(root_dir, 'build-wrapper-linux-x86/build-wrapper-linux-x86-64'),
                        '--out-dir', 'bw-outputs']
@@ -134,9 +146,14 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
                 flags += ['--cpu=arm64', '--cc-abi-flags=-arch arm64 -stdlib=libc++']
             else:
                 raise Exception("Unknown cross target '%s' for iOS" % (target))
-        elif target == 'cross-win32':
-            cc_bin = 'i686-w64-mingw32-g++'
-            flags += ['--cpu=x86_32', '--cc-abi-flags=-static', '--ar-command=i686-w64-mingw32-ar']
+        elif target == 'cross-i386':
+            flags += ['--cpu=x86_32']
+
+        elif target == 'cross-win64':
+            # MinGW in 16.04 is lacking std::mutex for unknown reason
+            cc_bin = 'x86_64-w64-mingw32-g++'
+            flags += ['--cpu=x86_64', '--cc-abi-flags=-static',
+                      '--ar-command=x86_64-w64-mingw32-ar', '--without-os-feature=threads']
             test_cmd = [os.path.join(root_dir, 'botan-test.exe')]
             # No runtime prefix required for Wine
         else:
@@ -158,21 +175,32 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
             elif target == 'cross-ppc64':
                 flags += ['--cpu=ppc64', '--with-endian=little']
                 cc_bin = 'powerpc64le-linux-gnu-g++'
-                test_prefix = ['qemu-ppc64le', '-L', '/usr/powerpc64le-linux-gnu/']
+                test_prefix = ['qemu-ppc64le', '-cpu', 'POWER8', '-L', '/usr/powerpc64le-linux-gnu/']
+            elif target == 'cross-mips64':
+                flags += ['--cpu=mips64', '--with-endian=big']
+                cc_bin = 'mips64-linux-gnuabi64-g++'
+                test_prefix = ['qemu-mips64', '-L', '/usr/mips64-linux-gnuabi64/']
             else:
                 raise Exception("Unknown cross target '%s' for Linux" % (target))
     else:
         # Flags specific to native targets
 
+        if target == 'gcc4.8':
+            cc_bin = 'g++-4.8'
+
         if target_os in ['osx', 'linux']:
             flags += ['--with-bzip2', '--with-sqlite', '--with-zlib']
 
+        if target_os in ['osx', 'ios']:
+            flags += ['--with-commoncrypto']
+
         if target_os == 'osx':
             # Test Boost on OS X
             flags += ['--with-boost']
             # Travis has 10.12 as default image
             flags += ['--with-os-features=getentropy']
-        elif target_os == 'linux':
+
+        if target_os == 'linux':
             flags += ['--with-lzma']
 
         if target_os == 'linux':
@@ -193,6 +221,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         flags += ['--cc-bin=%s' % (ccache)]
     else:
         flags += ['--cc-bin=%s %s' % (ccache, cc_bin)]
+        # Avoid putting the revision in build.h, which helps ccache hit rates
+        flags += ['--no-store-vc-rev']
 
     if test_cmd is None:
         run_test_command = None
@@ -228,7 +258,7 @@ def run_cmd(cmd, root_dir):
         print("Ran for %d seconds" % (time_taken))
 
     if proc.returncode != 0:
-        print("Command failed with error code %d" % (proc.returncode))
+        print("Command '%s' failed with error code %d" % (' '.join(cmd), proc.returncode))
 
         if cmd[0] not in ['lcov']:
             sys.exit(proc.returncode)
@@ -283,6 +313,11 @@ def parse_args(args):
     parser.add_option('--without-python3', dest='use_python3', action='store_false',
                       help='Disable using python3')
 
+    parser.add_option('--with-pylint3', dest='use_pylint3', action='store_true', default=True,
+                      help='Enable using python3 pylint')
+    parser.add_option('--without-pylint3', dest='use_pylint3', action='store_false',
+                      help='Disable using python3 pylint')
+
     return parser.parse_args(args)
 
 def have_prog(prog):
@@ -351,7 +386,7 @@ def main(args=None):
 
     root_dir = options.root_dir
 
-    if os.access(root_dir, os.R_OK) != True:
+    if not os.access(root_dir, os.R_OK):
         raise Exception('Bad root dir setting, dir %s not readable' % (root_dir))
 
     cmds = []
@@ -362,13 +397,17 @@ def main(args=None):
             raise Exception('No python interpreters found cannot lint')
 
         pylint_rc = '--rcfile=%s' % (os.path.join(root_dir, 'src/configs/pylint.rc'))
-        pylint_flags = [pylint_rc, '--reports=no', '--score=no']
+        pylint_flags = [pylint_rc, '--reports=no']
 
         # Some disabled rules specific to Python2
         # superfluous-parens: needed for Python3 compatible print statements
         # too-many-locals: variable counting differs from pylint3
         py2_flags = '--disable=superfluous-parens,too-many-locals'
 
+        # Some disabled rules specific to Python3
+        # useless-object-inheritance: complains about code still useful in Python2
+        py3_flags = '--disable=useless-object-inheritance'
+
         py_scripts = [
             'configure.py',
             'src/python/botan2.py',
@@ -380,18 +419,18 @@ def main(args=None):
             'src/scripts/website.py',
             'src/scripts/bench.py',
             'src/scripts/test_python.py',
+            'src/scripts/test_fuzzers.py',
             'src/scripts/test_cli.py',
             'src/scripts/python_unittests.py',
             'src/scripts/python_unittests_unix.py']
 
-        for target in py_scripts:
-            target_path = os.path.join(root_dir, target)
+        full_paths = [os.path.join(root_dir, s) for s in py_scripts]
 
-            if use_python2:
-                cmds.append(['python2', '-m', 'pylint'] + pylint_flags + [py2_flags, target_path])
+        if use_python2:
+            cmds.append(['python2', '-m', 'pylint'] + pylint_flags + [py2_flags] + full_paths)
 
-            if use_python3:
-                cmds.append(['python3', '-m', 'pylint'] + pylint_flags + [target_path])
+        if use_python3 and options.use_pylint3:
+            cmds.append(['python3', '-m', 'pylint'] + pylint_flags + [py3_flags] + full_paths)
 
     else:
         config_flags, run_test_command, make_prefix = determine_flags(
@@ -427,7 +466,7 @@ def main(args=None):
             elif options.compiler_cache == 'clcache':
                 cmds.append(['clcache', '-s'])
 
-        if run_test_command != None:
+        if run_test_command is not None:
             cmds.append(run_test_command)
 
         if target in ['coverage', 'fuzzers']:
