@@ -100,7 +100,7 @@ def gpg_sign(keyid, passphrase_file, files, detached=True):
     options = ['--armor', '--detach-sign'] if detached else ['--clearsign']
 
     gpg_cmd = ['gpg', '--batch'] + options + ['--local-user', keyid]
-    if passphrase_file != None:
+    if passphrase_file is not None:
         gpg_cmd[1:1] = ['--passphrase-file', passphrase_file]
 
     for filename in files:
@@ -248,7 +248,7 @@ def write_archive(output_basename, archive_type, rel_epoch, all_files, hash_file
     archive_hash = sha256.hexdigest().upper()
 
     logging.info('SHA-256(%s) = %s' % (output_archive, archive_hash))
-    if hash_file != None:
+    if hash_file is not None:
         hash_file.write("%s  %s\n" % (archive_hash, output_archive))
 
     return output_archive
@@ -259,7 +259,7 @@ def configure_logging(options):
             super(ExitOnErrorLogHandler, self).emit(record)
             # Exit script if and ERROR or worse occurred
             if record.levelno >= logging.ERROR:
-                if sys.exc_info()[2] != None:
+                if sys.exc_info()[2] is not None:
                     logging.info(traceback.format_exc())
                 sys.exit(1)
 
@@ -276,7 +276,7 @@ def configure_logging(options):
     logging.getLogger().setLevel(log_level())
 
 def main(args=None):
-    # pylint: disable=too-many-branches,too-many-locals
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
     if args is None:
         args = sys.argv[1:]
 
@@ -311,7 +311,7 @@ def main(args=None):
         try:
             logging.info('Creating release for version %s' % (target_version))
 
-            (major, minor, patch) = map(int, target_version.split('.'))
+            (major, minor, patch) = [int(x) for x in target_version.split('.')]
 
             assert target_version == '%d.%d.%d' % (major, minor, patch)
             target_version = target_version
@@ -355,14 +355,18 @@ def main(args=None):
         all_files += [os.path.join(curdir, f) for f in files]
     all_files.sort(key=lambda f: (os.path.dirname(f), os.path.basename(f)))
 
-    version_file = None
+    def find_version_file():
 
-    # location of file with version information has moved over time
-    for possible_version_file in ['src/build-data/version.txt', 'version.txt', 'botan_version.py']:
-        full_path = os.path.join(output_basename, possible_version_file)
-        if os.access(full_path, os.R_OK):
-            version_file = full_path
-            break
+        # location of file with version information has moved over time
+        for possible_version_file in ['src/build-data/version.txt', 'version.txt', 'botan_version.py']:
+            full_path = os.path.join(output_basename, possible_version_file)
+            if os.access(full_path, os.R_OK):
+                return full_path
+
+        logging.error('Cannot locate version file')
+        return None
+
+    version_file = find_version_file()
 
     if not os.access(version_file, os.R_OK):
         logging.error('Cannot read %s' % (version_file))
@@ -378,19 +382,19 @@ def main(args=None):
     output_files = []
 
     hash_file = None
-    if options.write_hash_file != None:
+    if options.write_hash_file is not None:
         hash_file = open(options.write_hash_file, 'w')
 
     for archive_type in archives:
         output_files.append(write_archive(output_basename, archive_type, rel_epoch, all_files, hash_file))
 
-    if hash_file != None:
+    if hash_file is not None:
         hash_file.close()
 
     shutil.rmtree(output_basename)
 
     if options.pgp_key_id != 'none':
-        if options.write_hash_file != None:
+        if options.write_hash_file is not None:
             output_files += gpg_sign(options.pgp_key_id, options.pgp_passphrase_file,
                                      [options.write_hash_file], False)
         else:
