@@ -60,7 +60,7 @@ int main(int argc, char* argv[])
       {
       const std::string arg_spec =
          "botan-test --verbose --help --data-dir= --pkcs11-lib= --provider= "
-         "--log-success --abort-on-first-fail --no-avoid-undefined "
+         "--log-success --abort-on-first-fail --no-avoid-undefined --skip-tests= "
          "--run-long-tests --run-online-tests --test-runs=1 --drbg-seed= "
          "*suites";
 
@@ -76,11 +76,13 @@ int main(int argc, char* argv[])
 
       const Botan_Tests::Test_Options opts(
          parser.get_arg_list("suites"),
+         parser.get_arg_list("skip-tests"),
          parser.get_arg_or("data-dir", "src/tests/data"),
          parser.get_arg("pkcs11-lib"),
          parser.get_arg("provider"),
          parser.get_arg("drbg-seed"),
          parser.get_arg_sz("test-runs"),
+         parser.flag_set("verbose"),
          parser.flag_set("log-success"),
          parser.flag_set("run-online-tests"),
          parser.flag_set("run-long-tests"),
@@ -96,7 +98,17 @@ int main(int argc, char* argv[])
 
       Botan_Tests::Test_Runner tests(std::cout);
 
-      return tests.run(opts);
+      int rc = tests.run(opts);
+
+#if defined(BOTAN_HAS_OPENSSL)
+      if(opts.provider().empty() || opts.provider() == "openssl")
+         {
+         ERR_free_strings();
+         ::ERR_remove_thread_state(nullptr);
+         }
+#endif
+
+      return rc;
       }
    catch(std::exception& e)
       {
