@@ -9,6 +9,9 @@
 #if defined(BOTAN_HAS_NUMBERTHEORY)
    #include <botan/bigint.h>
    #include <botan/numthry.h>
+   #include <botan/divide.h>
+   #include <botan/internal/primality.h>
+   #include <botan/reducer.h>
    #include <botan/pow_mod.h>
    #include <botan/parsing.h>
    #include "test_rng.h"
@@ -30,10 +33,10 @@ class BigInt_Unit_Tests final : public Test
          std::vector<Test::Result> results;
 
          results.push_back(test_bigint_sizes());
-         results.push_back(test_random_integer());
          results.push_back(test_random_prime());
          results.push_back(test_encode());
          results.push_back(test_bigint_io());
+         results.push_back(test_get_substring());
 
          return results;
          }
@@ -89,13 +92,13 @@ class BigInt_Unit_Tests final : public Test
          Test::Result result("BigInt prime generation");
 
          result.test_throws("Invalid bit size",
-                            "Invalid argument random_prime: Can't make a prime of 0 bits",
+                            "random_prime: Can't make a prime of 0 bits",
                             []() { Botan::random_prime(Test::rng(), 0); });
          result.test_throws("Invalid bit size",
-                            "Invalid argument random_prime: Can't make a prime of 1 bits",
+                            "random_prime: Can't make a prime of 1 bits",
                             []() { Botan::random_prime(Test::rng(), 1); });
          result.test_throws("Invalid arg",
-                            "Invalid argument random_prime Invalid value for equiv/modulo",
+                            "random_prime Invalid value for equiv/modulo",
                             []() { Botan::random_prime(Test::rng(), 2, 1, 0, 2); });
 
          BigInt p = Botan::random_prime(Test::rng(), 2);
@@ -133,51 +136,6 @@ class BigInt_Unit_Tests final : public Test
          return result;
          }
 
-      Test::Result test_random_integer()
-         {
-         Test::Result result("BigInt::random_integer");
-
-         result.start_timer();
-
-         const size_t SAMPLES = 500000;
-
-         const uint64_t range_min = 0;
-         const uint64_t range_max = 100;
-
-         /*
-         * We have a range of 0...100 thus 100 degrees of freedom.
-         * This bound is 99.9% probability of non-uniform
-         */
-         const double CHI_CRIT = 148.230;
-
-         std::map<uint32_t, size_t> counts;
-
-         for(size_t i = 0; i != SAMPLES; ++i)
-            {
-            uint32_t r = BigInt::random_integer(Test::rng(), range_min, range_max).to_u32bit();
-            counts[r] += 1;
-            }
-
-         // Chi-square test
-         const double expected = static_cast<double>(SAMPLES) / (range_max - range_min);
-         double chi2 = 0;
-
-         for(auto sample : counts)
-            {
-            const double count = sample.second;
-            chi2 += ((count - expected)*(count - expected)) / expected;
-            }
-
-         if(chi2 >= CHI_CRIT)
-            result.test_failure("Failed Chi-square test, value " + std::to_string(chi2));
-         else
-            result.test_success("Passed Chi-square test, value " + std::to_string(chi2));
-
-         result.end_timer();
-
-         return result;
-         }
-
       Test::Result test_encode()
          {
          Test::Result result("BigInt encoding functions");
@@ -204,6 +162,34 @@ class BigInt_Unit_Tests final : public Test
          return result;
          }
 
+      Test::Result test_get_substring()
+         {
+         const size_t trials = 1000;
+
+         Test::Result result("BigInt get_substring");
+
+         const Botan::BigInt r(Test::rng(), 250);
+
+         for(size_t s = 1; s <= 32; ++s)
+            {
+            for(size_t trial = 0; trial != trials; ++trial)
+               {
+               const size_t offset = Test::rng().next_byte();
+
+               const uint32_t val = r.get_substring(offset, s);
+
+               Botan::BigInt t = r >> offset;
+               t.mask_bits(s);
+
+               const uint32_t cmp = t.to_u32bit();
+
+               result.test_eq("Same value", size_t(val), size_t(cmp));
+               }
+            }
+
+         return result;
+         }
+
       Test::Result test_bigint_io()
          {
          Test::Result result("BigInt IO operators");
@@ -256,9 +242,9 @@ class BigInt_Add_Test final : public Text_Based_Test
 
          using Botan::BigInt;
 
-         const BigInt a = get_req_bn(vars, "In1");
-         const BigInt b = get_req_bn(vars, "In2");
-         const BigInt c = get_req_bn(vars, "Output");
+         const BigInt a = vars.get_req_bn("In1");
+         const BigInt b = vars.get_req_bn("In2");
+         const BigInt c = vars.get_req_bn("Output");
 
          result.test_eq("a + b", a + b, c);
          result.test_eq("b + a", b + a, c);
@@ -287,9 +273,9 @@ class BigInt_Sub_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Subtraction");
 
-         const BigInt a = get_req_bn(vars, "In1");
-         const BigInt b = get_req_bn(vars, "In2");
-         const BigInt c = get_req_bn(vars, "Output");
+         const BigInt a = vars.get_req_bn("In1");
+         const BigInt b = vars.get_req_bn("In2");
+         const BigInt c = vars.get_req_bn("Output");
 
          result.test_eq("a - b", a - b, c);
 
@@ -312,9 +298,9 @@ class BigInt_Mul_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Multiply");
 
-         const BigInt a = get_req_bn(vars, "In1");
-         const BigInt b = get_req_bn(vars, "In2");
-         const BigInt c = get_req_bn(vars, "Output");
+         const BigInt a = vars.get_req_bn("In1");
+         const BigInt b = vars.get_req_bn("In2");
+         const BigInt c = vars.get_req_bn("Output");
 
          result.test_eq("a * b", a * b, c);
          result.test_eq("b * a", b * a, c);
@@ -342,8 +328,8 @@ class BigInt_Sqr_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Square");
 
-         const BigInt input = get_req_bn(vars, "Input");
-         const BigInt output = get_req_bn(vars, "Output");
+         const BigInt input = vars.get_req_bn("Input");
+         const BigInt output = vars.get_req_bn("Output");
 
          result.test_eq("a * a", input * input, output);
          result.test_eq("sqr(a)", square(input), output);
@@ -363,9 +349,9 @@ class BigInt_Div_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Divide");
 
-         const BigInt a = get_req_bn(vars, "In1");
-         const BigInt b = get_req_bn(vars, "In2");
-         const BigInt c = get_req_bn(vars, "Output");
+         const BigInt a = vars.get_req_bn("In1");
+         const BigInt b = vars.get_req_bn("In2");
+         const BigInt c = vars.get_req_bn("Output");
 
          result.test_eq("a / b", a / b, c);
 
@@ -373,6 +359,20 @@ class BigInt_Div_Test final : public Text_Based_Test
          e /= b;
          result.test_eq("a /= b", e, c);
 
+         if(b.bytes() == 1)
+            {
+            const uint8_t b8 = b.byte_at(0);
+
+            Botan::BigInt ct_q;
+            uint8_t ct_r;
+            Botan::ct_divide_u8(a, b8, ct_q, ct_r);
+            result.test_eq("ct_divide_u8 q", ct_q, c);
+            }
+
+         Botan::BigInt ct_q, ct_r;
+         Botan::ct_divide(a, b, ct_q, ct_r);
+         result.test_eq("ct_divide q", ct_q, c);
+
          return result;
          }
    };
@@ -388,25 +388,43 @@ class BigInt_Mod_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Mod");
 
-         const BigInt a = get_req_bn(vars, "In1");
-         const BigInt b = get_req_bn(vars, "In2");
-         const BigInt c = get_req_bn(vars, "Output");
+         const BigInt a = vars.get_req_bn("In1");
+         const BigInt b = vars.get_req_bn("In2");
+         const BigInt expected = vars.get_req_bn("Output");
 
-         result.test_eq("a % b", a % b, c);
+         result.test_eq("a % b", a % b, expected);
 
          BigInt e = a;
          e %= b;
-         result.test_eq("a %= b", e, c);
+         result.test_eq("a %= b", e, expected);
+
+         const Botan::Modular_Reducer mod_b(b);
+         result.test_eq("Barrett", mod_b.reduce(a), expected);
 
          // if b fits into a Botan::word test %= operator for words
-         if(b.bytes() <= sizeof(Botan::word))
+         if(b.sig_words() == 1)
             {
-            Botan::word b_word = b.word_at(0);
+            const Botan::word b_word = b.word_at(0);
+
             e = a;
             e %= b_word;
-            result.test_eq("a %= b (as word)", e, c);
+            result.test_eq("a %= b (as word)", e, expected);
+
+            result.test_eq("a % b (as word)", a % b_word, expected);
             }
 
+         if(b.bytes() == 1)
+            {
+            Botan::BigInt ct_q;
+            Botan::uint8_t ct_r;
+            Botan::ct_divide_u8(a, b.byte_at(0), ct_q, ct_r);
+            result.test_eq("ct_divide_u8 r", ct_r, expected);
+            }
+
+         Botan::BigInt ct_q, ct_r;
+         Botan::ct_divide(a, b, ct_q, ct_r);
+         result.test_eq("ct_divide r", ct_r, expected);
+
          return result;
          }
    };
@@ -422,9 +440,9 @@ class BigInt_GCD_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Mod");
 
-         const BigInt x = get_req_bn(vars, "X");
-         const BigInt y = get_req_bn(vars, "Y");
-         const BigInt expected = get_req_bn(vars, "GCD");
+         const BigInt x = vars.get_req_bn("X");
+         const BigInt y = vars.get_req_bn("Y");
+         const BigInt expected = vars.get_req_bn("GCD");
 
          const BigInt g = Botan::gcd(x, y);
 
@@ -444,9 +462,9 @@ class BigInt_Lshift_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Lshift");
 
-         const BigInt value = get_req_bn(vars, "Value");
-         const size_t shift = get_req_bn(vars, "Shift").to_u32bit();
-         const BigInt output = get_req_bn(vars, "Output");
+         const BigInt value = vars.get_req_bn("Value");
+         const size_t shift = vars.get_req_bn("Shift").to_u32bit();
+         const BigInt output = vars.get_req_bn("Output");
 
          result.test_eq("a << s", value << shift, output);
 
@@ -469,9 +487,9 @@ class BigInt_Rshift_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Rshift");
 
-         const BigInt value = get_req_bn(vars, "Value");
-         const size_t shift = get_req_bn(vars, "Shift").to_u32bit();
-         const BigInt output = get_req_bn(vars, "Output");
+         const BigInt value = vars.get_req_bn("Value");
+         const size_t shift = vars.get_req_bn("Shift").to_u32bit();
+         const BigInt output = vars.get_req_bn("Output");
 
          result.test_eq("a >> s", value >> shift, output);
 
@@ -494,10 +512,10 @@ class BigInt_Powmod_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Powmod");
 
-         const BigInt base = get_req_bn(vars, "Base");
-         const BigInt exponent = get_req_bn(vars, "Exponent");
-         const BigInt modulus = get_req_bn(vars, "Modulus");
-         const BigInt expected = get_req_bn(vars, "Output");
+         const BigInt base = vars.get_req_bn("Base");
+         const BigInt exponent = vars.get_req_bn("Exponent");
+         const BigInt modulus = vars.get_req_bn("Modulus");
+         const BigInt expected = vars.get_req_bn("Output");
 
          result.test_eq("power_mod", Botan::power_mod(base, exponent, modulus), expected);
 
@@ -550,17 +568,37 @@ class BigInt_IsPrime_Test final : public Text_Based_Test
             throw Test_Error("Bad header for prime test " + header);
             }
 
-         const BigInt value = get_req_bn(vars, "X");
+         const BigInt value = vars.get_req_bn("X");
          const bool is_prime = (header == "Prime");
 
          Test::Result result("BigInt Test " + header);
          result.test_eq("is_prime", Botan::is_prime(value, Test::rng()), is_prime);
+
          return result;
          }
    };
 
 BOTAN_REGISTER_TEST("bn_isprime", BigInt_IsPrime_Test);
 
+class BigInt_IsSquare_Test final : public Text_Based_Test
+   {
+   public:
+      BigInt_IsSquare_Test() : Text_Based_Test("bn/perfect_square.vec", "X,R") {}
+
+      Test::Result run_one_test(const std::string&, const VarMap& vars) override
+         {
+         const BigInt value = vars.get_req_bn("X");
+         const BigInt expected = vars.get_req_bn("R");
+         const BigInt computed = Botan::is_perfect_square(value);
+
+         Test::Result result("BigInt IsSquare");
+         result.test_eq("is_perfect_square", computed, expected);
+         return result;
+         }
+   };
+
+BOTAN_REGISTER_TEST("bn_issquare", BigInt_IsSquare_Test);
+
 class BigInt_Ressol_Test final : public Text_Based_Test
    {
    public:
@@ -570,9 +608,9 @@ class BigInt_Ressol_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Ressol");
 
-         const Botan::BigInt a = get_req_bn(vars, "Input");
-         const Botan::BigInt p = get_req_bn(vars, "Modulus");
-         const Botan::BigInt exp = get_req_bn(vars, "Output");
+         const Botan::BigInt a = vars.get_req_bn("Input");
+         const Botan::BigInt p = vars.get_req_bn("Modulus");
+         const Botan::BigInt exp = vars.get_req_bn("Output");
 
          const Botan::BigInt a_sqrt = Botan::ressol(a, p);
 
@@ -599,9 +637,9 @@ class BigInt_InvMod_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt InvMod");
 
-         const Botan::BigInt a = get_req_bn(vars, "Input");
-         const Botan::BigInt mod = get_req_bn(vars, "Modulus");
-         const Botan::BigInt expected = get_req_bn(vars, "Output");
+         const Botan::BigInt a = vars.get_req_bn("Input");
+         const Botan::BigInt mod = vars.get_req_bn("Modulus");
+         const Botan::BigInt expected = vars.get_req_bn("Output");
 
          const Botan::BigInt a_inv = Botan::inverse_euclid(a, mod);
 
@@ -641,10 +679,10 @@ class BigInt_Rand_Test final : public Text_Based_Test
          {
          Test::Result result("BigInt Random");
 
-         const std::vector<uint8_t> seed = get_req_bin(vars, "Seed");
-         const Botan::BigInt min = get_req_bn(vars, "Min");
-         const Botan::BigInt max = get_req_bn(vars, "Max");
-         const Botan::BigInt expected = get_req_bn(vars, "Output");
+         const std::vector<uint8_t> seed = vars.get_req_bin("Seed");
+         const Botan::BigInt min = vars.get_req_bn("Min");
+         const Botan::BigInt max = vars.get_req_bn("Max");
+         const Botan::BigInt expected = vars.get_req_bn("Output");
 
          Fixed_Output_RNG rng(seed);
          Botan::BigInt generated = BigInt::random_integer(rng, min, max);
@@ -657,6 +695,47 @@ class BigInt_Rand_Test final : public Text_Based_Test
 
 BOTAN_REGISTER_TEST("bn_rand", BigInt_Rand_Test);
 
+class Lucas_Primality_Test final : public Test
+   {
+   public:
+
+      std::vector<Test::Result> run() override
+         {
+         const uint32_t lucas_max = (Test::run_long_tests() ? 100000 : 6000);
+
+         // OEIS A217120
+         std::set<uint32_t> lucas_pp{
+            323, 377, 1159, 1829, 3827, 5459, 5777, 9071, 9179,
+            10877, 11419, 11663, 13919, 14839, 16109, 16211, 18407, 18971,
+            19043, 22499, 23407, 24569, 25199, 25877, 26069, 27323, 32759,
+            34943, 35207, 39059, 39203, 39689, 40309, 44099, 46979, 47879,
+            50183, 51983, 53663, 56279, 58519, 60377, 63881, 69509, 72389,
+            73919, 75077, 77219, 79547, 79799, 82983, 84419, 86063, 90287,
+            94667, 97019, 97439,
+         };
+
+         Test::Result result("Lucas primality test");
+
+         for(uint32_t i = 3; i <= lucas_max; i += 2)
+            {
+            Botan::Modular_Reducer mod_i(i);
+            const bool passes_lucas = Botan::is_lucas_probable_prime(i, mod_i);
+            const bool is_prime = Botan::is_prime(i, Test::rng());
+
+            const bool is_lucas_pp = (is_prime == false && passes_lucas == true);
+
+            if(is_lucas_pp)
+               result.confirm("Lucas pseudoprime is in list", lucas_pp.count(i) == 1);
+            else
+               result.confirm("Lucas non-pseudoprime is not in list", lucas_pp.count(i) == 0);
+            }
+
+         return {result};
+         }
+   };
+
+BOTAN_REGISTER_TEST("bn_lucas", Lucas_Primality_Test);
+
 class DSA_ParamGen_Test final : public Text_Based_Test
    {
    public:
@@ -664,11 +743,11 @@ class DSA_ParamGen_Test final : public Text_Based_Test
 
       Test::Result run_one_test(const std::string& header, const VarMap& vars) override
          {
-         const std::vector<uint8_t> seed = get_req_bin(vars, "Seed");
-         const size_t offset = get_req_sz(vars, "Counter");
+         const std::vector<uint8_t> seed = vars.get_req_bin("Seed");
+         const size_t offset = vars.get_req_sz("Counter");
 
-         const Botan::BigInt exp_P = get_req_bn(vars, "P");
-         const Botan::BigInt exp_Q = get_req_bn(vars, "Q");
+         const Botan::BigInt exp_P = vars.get_req_bn("P");
+         const Botan::BigInt exp_Q = vars.get_req_bn("Q");
 
          const std::vector<std::string> header_parts = Botan::split_on(header, ',');
 
@@ -682,12 +761,6 @@ class DSA_ParamGen_Test final : public Text_Based_Test
 
          Test::Result result("DSA Parameter Generation");
 
-         // These tests are very slow so skip in normal runs
-         if(p_bits > 1024 && Test::run_long_tests() == false)
-            {
-            return result;
-            }
-
          try
             {
             Botan::BigInt gen_P, gen_Q;
