@@ -15,7 +15,7 @@ namespace Botan_Tests {
 class Block_Cipher_Tests final : public Text_Based_Test
    {
    public:
-      Block_Cipher_Tests() : Text_Based_Test("block", "Key,In,Out", "Iterations") {}
+      Block_Cipher_Tests() : Text_Based_Test("block", "Key,In,Out", "Tweak,Iterations") {}
 
       std::vector<std::string> possible_providers(const std::string& algo) override
          {
@@ -24,10 +24,11 @@ class Block_Cipher_Tests final : public Text_Based_Test
 
       Test::Result run_one_test(const std::string& algo, const VarMap& vars) override
          {
-         const std::vector<uint8_t> key      = get_req_bin(vars, "Key");
-         const std::vector<uint8_t> input    = get_req_bin(vars, "In");
-         const std::vector<uint8_t> expected = get_req_bin(vars, "Out");
-         const size_t iterations             = get_opt_sz(vars, "Iterations", 1);
+         const std::vector<uint8_t> key      = vars.get_req_bin("Key");
+         const std::vector<uint8_t> input    = vars.get_req_bin("In");
+         const std::vector<uint8_t> expected = vars.get_req_bin("Out");
+         const std::vector<uint8_t> tweak    = vars.get_opt_bin("Tweak");
+         const size_t iterations             = vars.get_opt_sz("Iterations", 1);
 
          Test::Result result(algo);
 
@@ -85,8 +86,28 @@ class Block_Cipher_Tests final : public Text_Based_Test
             cipher->encrypt(garbage);
             cipher->clear();
 
+            /*
+            * Different providers may have additional restrictions on key sizes.
+            * Avoid testing the cipher with a key size that it does not natively support.
+            */
+            if(!cipher->valid_keylength(key.size()))
+               {
+               result.test_note("Skipping test with provider " + provider +
+                                " as it does not support key length " + std::to_string(key.size()));
+               continue;
+               }
+
             cipher->set_key(key);
 
+            if(tweak.size() > 0)
+               {
+               Botan::Tweakable_Block_Cipher* tbc = dynamic_cast<Botan::Tweakable_Block_Cipher*>(cipher.get());
+               if(tbc == nullptr)
+                  result.test_failure("Tweak set in test data but cipher is not a Tweakable_Block_Cipher");
+               else
+                  tbc->set_tweak(tweak.data(), tweak.size());
+               }
+
             // Test that clone works and does not affect parent object
             std::unique_ptr<Botan::BlockCipher> clone(cipher->clone());
             result.confirm("Clone has different pointer", cipher.get() != clone.get());
@@ -116,7 +137,7 @@ class Block_Cipher_Tests final : public Text_Based_Test
             // Now test misaligned buffers
             const size_t blocks = input.size() / cipher->block_size();
             buf.resize(input.size() + 1);
-            std::memcpy(buf.data() + 1, input.data(), input.size());
+            Botan::copy_mem(buf.data() + 1, input.data(), input.size());
 
             for(size_t i = 0; i != iterations; ++i)
                {
@@ -128,7 +149,7 @@ class Block_Cipher_Tests final : public Text_Based_Test
                            expected.data(), expected.size());
 
             // always decrypt expected ciphertext vs what we produced above
-            std::memcpy(buf.data() + 1, expected.data(), expected.size());
+            Botan::copy_mem(buf.data() + 1, expected.data(), expected.size());
 
             for(size_t i = 0; i != iterations; ++i)
                {
