@@ -36,9 +36,9 @@ class ECDSA_Verification_Tests final : public PK_Signature_Verification_Test
 
       std::unique_ptr<Botan::Public_Key> load_public_key(const VarMap& vars) override
          {
-         const std::string group_id = get_req_str(vars, "Group");
-         const BigInt px = get_req_bn(vars, "Px");
-         const BigInt py = get_req_bn(vars, "Py");
+         const std::string group_id = vars.get_req_str("Group");
+         const BigInt px = vars.get_req_bn("Px");
+         const BigInt py = vars.get_req_bn("Py");
          Botan::EC_Group group(Botan::OIDS::lookup(group_id));
 
          const Botan::PointGFp public_point = group.point(px, py);
@@ -53,6 +53,46 @@ class ECDSA_Verification_Tests final : public PK_Signature_Verification_Test
          }
    };
 
+class ECDSA_Wycheproof_Verification_Tests final : public PK_Signature_Verification_Test
+   {
+   public:
+      ECDSA_Wycheproof_Verification_Tests() : PK_Signature_Verification_Test(
+            "ECDSA",
+            "pubkey/ecdsa_wycheproof.vec",
+            "Group,Px,Py,Hash,Msg,Signature",
+            "Valid") {}
+
+      bool clear_between_callbacks() const override
+         {
+         return false;
+         }
+
+      Botan::Signature_Format sig_format() const override
+         {
+         return Botan::DER_SEQUENCE;
+         }
+
+      bool test_random_invalid_sigs() const override { return false; }
+
+      std::unique_ptr<Botan::Public_Key> load_public_key(const VarMap& vars) override
+         {
+         const std::string group_id = vars.get_req_str("Group");
+         const BigInt px = vars.get_req_bn("Px");
+         const BigInt py = vars.get_req_bn("Py");
+         Botan::EC_Group group(Botan::OIDS::lookup(group_id));
+
+         const Botan::PointGFp public_point = group.point(px, py);
+
+         std::unique_ptr<Botan::Public_Key> key(new Botan::ECDSA_PublicKey(group, public_point));
+         return key;
+         }
+
+      std::string default_padding(const VarMap& vars) const override
+         {
+         return "EMSA1(" + vars.get_req_str("Hash") + ")";
+         }
+   };
+
 class ECDSA_Signature_KAT_Tests final : public PK_Signature_Generation_Test
    {
    public:
@@ -73,8 +113,8 @@ class ECDSA_Signature_KAT_Tests final : public PK_Signature_Generation_Test
 
       std::unique_ptr<Botan::Private_Key> load_private_key(const VarMap& vars) override
          {
-         const std::string group_id = get_req_str(vars, "Group");
-         const BigInt x = get_req_bn(vars, "X");
+         const std::string group_id = vars.get_req_str("Group");
+         const BigInt x = vars.get_req_bn("X");
          Botan::EC_Group group(Botan::OIDS::lookup(group_id));
 
          std::unique_ptr<Botan::Private_Key> key(new Botan::ECDSA_PrivateKey(Test::rng(), group, x));
@@ -83,7 +123,7 @@ class ECDSA_Signature_KAT_Tests final : public PK_Signature_Generation_Test
 
       std::string default_padding(const VarMap& vars) const override
          {
-         const std::string hash = get_req_str(vars, "Hash");
+         const std::string hash = vars.get_req_str("Hash");
          if(hash.substr(0,3) == "Raw")
             return hash;
          return "EMSA1(" + hash + ")";
@@ -112,6 +152,56 @@ class ECDSA_Keygen_Tests final : public PK_Key_Generation_Test
          }
    };
 
+#if defined(BOTAN_HAS_EMSA_RAW)
+
+class ECDSA_Key_Recovery_Tests final : public Text_Based_Test
+   {
+   public:
+      ECDSA_Key_Recovery_Tests() :
+         Text_Based_Test("pubkey/ecdsa_key_recovery.vec", "Group,Msg,R,S,V,PubkeyX,PubkeyY") {}
+
+      Test::Result run_one_test(const std::string&, const VarMap& vars) override
+         {
+         Test::Result result("ECDSA key recovery");
+
+         const std::string group_id = vars.get_req_str("Group");
+         Botan::EC_Group group(group_id);
+
+         const BigInt R = vars.get_req_bn("R");
+         const BigInt S = vars.get_req_bn("S");
+         const uint8_t V = vars.get_req_u8("V");
+         const std::vector<uint8_t> msg = vars.get_req_bin("Msg");
+         const BigInt pubkey_x = vars.get_req_bn("PubkeyX");
+         const BigInt pubkey_y = vars.get_req_bn("PubkeyY");
+
+         try
+            {
+            Botan::ECDSA_PublicKey pubkey(group, msg, R, S, V);
+            result.test_eq("Pubkey X coordinate", pubkey.public_point().get_affine_x(), pubkey_x);
+            result.test_eq("Pubkey Y coordinate", pubkey.public_point().get_affine_y(), pubkey_y);
+
+            const uint8_t computed_V = pubkey.recovery_param(msg, R, S);
+            result.test_eq("Recovery param is correct", static_cast<size_t>(computed_V), static_cast<size_t>(V));
+
+            Botan::PK_Verifier verifier(pubkey, "Raw");
+
+            auto sig = Botan::BigInt::encode_fixed_length_int_pair(R, S, group.get_order_bytes());
+
+            result.confirm("Signature verifies", verifier.verify_message(msg, sig));
+            }
+         catch(Botan::Exception& e)
+            {
+            result.test_failure("Failed to recover ECDSA public key", e.what());
+            }
+
+         return result;
+         }
+   };
+
+BOTAN_REGISTER_TEST("ecdsa_key_recovery", ECDSA_Key_Recovery_Tests);
+
+#endif
+
 class ECDSA_Invalid_Key_Tests final : public Text_Based_Test
    {
    public:
@@ -127,10 +217,10 @@ class ECDSA_Invalid_Key_Tests final : public Text_Based_Test
          {
          Test::Result result("ECDSA invalid keys");
 
-         const std::string group_id = get_req_str(vars, "Group");
+         const std::string group_id = vars.get_req_str("Group");
          Botan::EC_Group group(Botan::OIDS::lookup(group_id));
-         const Botan::BigInt x = get_req_bn(vars, "InvalidKeyX");
-         const Botan::BigInt y = get_req_bn(vars, "InvalidKeyY");
+         const Botan::BigInt x = vars.get_req_bn("InvalidKeyX");
+         const Botan::BigInt y = vars.get_req_bn("InvalidKeyY");
 
          std::unique_ptr<Botan::PointGFp> public_point;
 
@@ -154,6 +244,7 @@ class ECDSA_Invalid_Key_Tests final : public Text_Based_Test
    };
 
 BOTAN_REGISTER_TEST("ecdsa_verify", ECDSA_Verification_Tests);
+BOTAN_REGISTER_TEST("ecdsa_verify_wycheproof", ECDSA_Wycheproof_Verification_Tests);
 BOTAN_REGISTER_TEST("ecdsa_sign", ECDSA_Signature_KAT_Tests);
 BOTAN_REGISTER_TEST("ecdsa_keygen", ECDSA_Keygen_Tests);
 BOTAN_REGISTER_TEST("ecdsa_invalid", ECDSA_Invalid_Key_Tests);
