@@ -47,7 +47,7 @@ class Invalid_Hash_Name_Tests final : public Test
          catch(Botan::Invalid_Argument& e)
             {
             const std::string msg = e.what();
-            const std::string full_msg = "Invalid argument " + expected_msg;
+            const std::string full_msg = "" + expected_msg;
             result.test_eq("expected error message", msg, full_msg);
             }
          catch(Botan::Lookup_Error& e)
@@ -81,8 +81,8 @@ class Hash_Function_Tests final : public Text_Based_Test
 
       Test::Result run_one_test(const std::string& algo, const VarMap& vars) override
          {
-         const std::vector<uint8_t> input    = get_req_bin(vars, "In");
-         const std::vector<uint8_t> expected = get_req_bin(vars, "Out");
+         const std::vector<uint8_t> input    = vars.get_req_bin("In");
+         const std::vector<uint8_t> expected = vars.get_req_bin("Out");
 
          Test::Result result(algo);
 
@@ -125,6 +125,22 @@ class Hash_Function_Tests final : public Text_Based_Test
 
             result.test_eq(provider, "hashing after clear", hash->final(), expected);
 
+            // Test that misaligned inputs work
+
+            if(input.size() > 0)
+               {
+               std::vector<uint8_t> misaligned = input;
+               const size_t current_alignment = reinterpret_cast<uintptr_t>(misaligned.data()) % 16;
+
+               const size_t bytes_to_misalign = 15 - current_alignment;
+
+               for(size_t i = 0; i != bytes_to_misalign; ++i)
+                  misaligned.insert(misaligned.begin(), 0x23);
+
+               hash->update(&misaligned[bytes_to_misalign], input.size());
+               result.test_eq(provider, "hashing misaligned data", hash->final(), expected);
+               }
+
             if(input.size() > 5)
                {
                hash->update(input[0]);
