@@ -88,19 +88,19 @@ std::string PK_Test::choose_padding(const VarMap& vars,
    {
    if(pad_hdr != "")
       return pad_hdr;
-   return get_opt_str(vars, "Padding", this->default_padding(vars));
+   return vars.get_opt_str("Padding", this->default_padding(vars));
    }
 
 std::vector<std::string> PK_Test::possible_providers(const std::string& /*params*/)
    {
-   return Test::provider_filter({ "base", "bearssl", "openssl", "tpm" });
+   return Test::provider_filter({ "base", "commoncrypto", "bearssl", "openssl", "tpm" });
    }
 
 Test::Result
 PK_Signature_Generation_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
-   const std::vector<uint8_t> message   = get_req_bin(vars, "Msg");
-   const std::vector<uint8_t> signature = get_req_bin(vars, "Signature");
+   const std::vector<uint8_t> message   = vars.get_req_bin("Msg");
+   const std::vector<uint8_t> signature = vars.get_req_bin("Signature");
    const std::string padding = choose_padding(vars, pad_hdr);
 
    Test::Result result(algo_name() + "/" + padding + " signature generation");
@@ -143,9 +143,9 @@ PK_Signature_Generation_Test::run_one_test(const std::string& pad_hdr, const Var
    for(auto const& sign_provider : possible_providers(algo_name()))
       {
       std::unique_ptr<Botan::RandomNumberGenerator> rng;
-      if(vars.count("Nonce"))
+      if(vars.has_key("Nonce"))
          {
-         rng.reset(test_rng(get_req_bin(vars, "Nonce")));
+         rng.reset(test_rng(vars.get_req_bin("Nonce")));
          }
 
       std::unique_ptr<Botan::PK_Signer> signer;
@@ -157,6 +157,9 @@ PK_Signature_Generation_Test::run_one_test(const std::string& pad_hdr, const Var
          signer.reset(new Botan::PK_Signer(*privkey, Test::rng(), padding, Botan::IEEE_1363, sign_provider));
 
          generated_signature = signer->sign_message(message, rng ? *rng : Test::rng());
+
+         result.test_lte("Generated signature within announced bound",
+                         generated_signature.size(), signer->signature_length());
          }
       catch(Botan::Lookup_Error&)
          {
@@ -184,14 +187,20 @@ PK_Signature_Generation_Test::run_one_test(const std::string& pad_hdr, const Var
    return result;
    }
 
+Botan::Signature_Format
+PK_Signature_Verification_Test::sig_format() const
+   {
+   return Botan::IEEE_1363;
+   }
+
 Test::Result
 PK_Signature_Verification_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
-   const std::vector<uint8_t> message   = get_req_bin(vars, "Msg");
-   const std::vector<uint8_t> signature = get_req_bin(vars, "Signature");
+   const std::vector<uint8_t> message   = vars.get_req_bin("Msg");
+   const std::vector<uint8_t> signature = vars.get_req_bin("Signature");
    const std::string padding = choose_padding(vars, pad_hdr);
 
-   const bool expected_valid = (get_opt_sz(vars, "Valid", 1) == 1);
+   const bool expected_valid = (vars.get_opt_sz("Valid", 1) == 1);
 
    std::unique_ptr<Botan::Public_Key> pubkey = load_public_key(vars);
 
@@ -203,24 +212,35 @@ PK_Signature_Verification_Test::run_one_test(const std::string& pad_hdr, const V
 
       try
          {
-         verifier.reset(new Botan::PK_Verifier(*pubkey, padding, Botan::IEEE_1363, verify_provider));
+         verifier.reset(new Botan::PK_Verifier(*pubkey, padding, sig_format(), verify_provider));
          }
       catch(Botan::Lookup_Error&)
          {
-         result.test_note("Skipping verifying with " + verify_provider);
+         //result.test_note("Skipping verifying with " + verify_provider);
          }
 
       if(verifier)
          {
-         const bool verified = verifier->verify_message(message, signature);
-
-         if(expected_valid)
+         try
             {
-            result.test_eq("correct signature valid", verified, true);
-            check_invalid_signatures(result, *verifier, message, signature);
+            const bool verified = verifier->verify_message(message, signature);
+
+            if(expected_valid)
+               {
+               result.test_eq("correct signature valid with " + verify_provider, verified, true);
+
+               if(test_random_invalid_sigs())
+                  {
+                  check_invalid_signatures(result, *verifier, message, signature);
+                  }
+               }
+            else
+               result.test_eq("incorrect signature invalid", verified, false);
+            }
+         catch(std::exception& e)
+            {
+            result.test_failure("verification threw exception", e.what());
             }
-         else
-            result.test_eq("incorrect signature invalid", verified, false);
          }
       }
 
@@ -231,10 +251,10 @@ Test::Result
 PK_Signature_NonVerification_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
    const std::string padding = choose_padding(vars, pad_hdr);
-   const std::vector<uint8_t> message   = get_req_bin(vars, "Msg");
+   const std::vector<uint8_t> message   = vars.get_req_bin("Msg");
    std::unique_ptr<Botan::Public_Key> pubkey = load_public_key(vars);
 
-   const std::vector<uint8_t> invalid_signature = get_req_bin(vars, "InvalidSignature");
+   const std::vector<uint8_t> invalid_signature = vars.get_req_bin("InvalidSignature");
 
    Test::Result result(algo_name() + "/" + padding + " verify invalid signature");
 
@@ -259,8 +279,8 @@ PK_Signature_NonVerification_Test::run_one_test(const std::string& pad_hdr, cons
 Test::Result
 PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
-   const std::vector<uint8_t> plaintext  = get_req_bin(vars, "Msg");
-   const std::vector<uint8_t> ciphertext = get_req_bin(vars, "Ciphertext");
+   const std::vector<uint8_t> plaintext  = vars.get_req_bin("Msg");
+   const std::vector<uint8_t> ciphertext = vars.get_req_bin("Ciphertext");
    const std::string padding = choose_padding(vars, pad_hdr);
 
    Test::Result result(algo_name() + (padding.empty() ? padding : "/" + padding) + " encryption");
@@ -290,6 +310,10 @@ PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const Va
       try
          {
          decrypted = decryptor->decrypt(ciphertext);
+
+         result.test_lte("Plaintext within length",
+                         decrypted.size(),
+                         decryptor->plaintext_length(ciphertext.size()));
          }
       catch(Botan::Exception& e)
          {
@@ -315,9 +339,9 @@ PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const Va
          }
 
       std::unique_ptr<Botan::RandomNumberGenerator> kat_rng;
-      if(vars.count("Nonce"))
+      if(vars.has_key("Nonce"))
          {
-         kat_rng.reset(test_rng(get_req_bin(vars, "Nonce")));
+         kat_rng.reset(test_rng(vars.get_req_bin("Nonce")));
          }
 
       if(padding == "Raw")
@@ -338,6 +362,10 @@ PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const Va
       const std::vector<uint8_t> generated_ciphertext =
          encryptor->encrypt(plaintext, kat_rng ? *kat_rng : Test::rng());
 
+      result.test_lte("Ciphertext within length",
+                      generated_ciphertext.size(),
+                      encryptor->ciphertext_length(plaintext.size()));
+
       if(enc_provider == "base")
          {
          result.test_eq(enc_provider, "generated ciphertext matches KAT",
@@ -360,8 +388,8 @@ PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const Va
 Test::Result
 PK_Decryption_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
-   const std::vector<uint8_t> plaintext  = get_req_bin(vars, "Msg");
-   const std::vector<uint8_t> ciphertext = get_req_bin(vars, "Ciphertext");
+   const std::vector<uint8_t> plaintext  = vars.get_req_bin("Msg");
+   const std::vector<uint8_t> ciphertext = vars.get_req_bin("Ciphertext");
    const std::string padding = choose_padding(vars, pad_hdr);
 
    Test::Result result(algo_name() + (padding.empty() ? padding : "/" + padding) + " decryption");
@@ -402,10 +430,10 @@ PK_Decryption_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
 
 Test::Result PK_KEM_Test::run_one_test(const std::string&, const VarMap& vars)
    {
-   const std::vector<uint8_t> K = get_req_bin(vars, "K");
-   const std::vector<uint8_t> C0 = get_req_bin(vars, "C0");
-   const std::vector<uint8_t> salt = get_opt_bin(vars, "Salt");
-   const std::string kdf = get_req_str(vars, "KDF");
+   const std::vector<uint8_t> K = vars.get_req_bin("K");
+   const std::vector<uint8_t> C0 = vars.get_req_bin("C0");
+   const std::vector<uint8_t> salt = vars.get_opt_bin("Salt");
+   const std::string kdf = vars.get_req_str("KDF");
 
    Test::Result result(algo_name() + "/" + kdf + " KEM");
 
@@ -426,7 +454,7 @@ Test::Result PK_KEM_Test::run_one_test(const std::string&, const VarMap& vars)
       return result;
       }
 
-   Fixed_Output_RNG fixed_output_rng(get_req_bin(vars, "R"));
+   Fixed_Output_RNG fixed_output_rng(vars.get_req_bin("R"));
 
    Botan::secure_vector<uint8_t> produced_encap_key, shared_key;
    enc->encrypt(produced_encap_key,
@@ -462,8 +490,8 @@ Test::Result PK_KEM_Test::run_one_test(const std::string&, const VarMap& vars)
 
 Test::Result PK_Key_Agreement_Test::run_one_test(const std::string& header, const VarMap& vars)
    {
-   const std::vector<uint8_t> shared = get_req_bin(vars, "K");
-   const std::string kdf = get_opt_str(vars, "KDF", default_kdf(vars));
+   const std::vector<uint8_t> shared = vars.get_req_bin("K");
+   const std::string kdf = vars.get_opt_str("KDF", default_kdf(vars));
 
    Test::Result result(algo_name() + "/" + kdf +
                        (header.empty() ? header : " " + header) +
@@ -472,7 +500,7 @@ Test::Result PK_Key_Agreement_Test::run_one_test(const std::string& header, cons
    std::unique_ptr<Botan::Private_Key> privkey = load_our_key(header, vars);
    const std::vector<uint8_t> pubkey = load_their_key(header, vars);
 
-   const size_t key_len = get_opt_sz(vars, "OutLen", 0);
+   const size_t key_len = vars.get_opt_sz("OutLen", 0);
 
    for(auto const& provider : possible_providers(algo_name()))
       {
@@ -481,7 +509,14 @@ Test::Result PK_Key_Agreement_Test::run_one_test(const std::string& header, cons
       try
          {
          kas.reset(new Botan::PK_Key_Agreement(*privkey, Test::rng(), kdf, provider));
-         result.test_eq(provider, "agreement", kas->derive_key(key_len, pubkey).bits_of(), shared);
+
+         auto derived_key = kas->derive_key(key_len, pubkey).bits_of();
+         result.test_eq(provider, "agreement", derived_key, shared);
+
+         if(key_len == 0 && kdf == "Raw")
+            {
+            result.test_eq("Expected size", derived_key.size(), kas->agreed_value_size());
+            }
          }
       catch(Botan::Lookup_Error&)
          {
@@ -496,10 +531,67 @@ std::vector<std::string> PK_Key_Generation_Test::possible_providers(
    const std::string& algo)
    {
    std::vector<std::string> pk_provider =
-      Botan::probe_provider_private_key(algo, { "base", "openssl", "tpm" });
+      Botan::probe_provider_private_key(algo, { "base", "commoncrypto", "openssl", "tpm" });
    return Test::provider_filter(pk_provider);
    }
 
+namespace {
+
+void test_pbe_roundtrip(Test::Result& result,
+                        const Botan::Private_Key& key,
+                        const std::string& pbe_algo,
+                        const std::string& passphrase)
+   {
+   try
+      {
+      Botan::DataSource_Memory data_src(
+         Botan::PKCS8::PEM_encode(key, Test::rng(), passphrase,
+                                  std::chrono::milliseconds(10),
+                                  pbe_algo));
+
+      std::unique_ptr<Botan::Private_Key> loaded(
+         Botan::PKCS8::load_key(data_src, Test::rng(), passphrase));
+
+      result.confirm("recovered private key from encrypted blob", loaded.get() != nullptr);
+      result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
+      try
+         {
+         result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
+         }
+      catch(Botan::Lookup_Error&) {}
+      }
+   catch(std::exception& e)
+      {
+      result.test_failure("roundtrip encrypted PEM private key", e.what());
+      }
+
+   try
+      {
+      Botan::DataSource_Memory data_src(
+         Botan::PKCS8::BER_encode(key, Test::rng(), passphrase,
+                                  std::chrono::milliseconds(10),
+                                  pbe_algo));
+
+      std::unique_ptr<Botan::Private_Key> loaded(
+         Botan::PKCS8::load_key(data_src, Test::rng(), passphrase));
+
+      result.confirm("recovered private key from BER blob", loaded.get() != nullptr);
+      result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
+
+      try
+         {
+         result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
+         }
+      catch(Botan::Lookup_Error&) {}
+      }
+   catch(std::exception& e)
+      {
+      result.test_failure("roundtrip encrypted BER private key", e.what());
+      }
+   }
+
+}
+
 std::vector<Test::Result> PK_Key_Generation_Test::run()
    {
    std::vector<Test::Result> results;
@@ -613,58 +705,16 @@ std::vector<Test::Result> PK_Key_Generation_Test::run()
             result.test_failure("roundtrip BER private key", e.what());
             }
 
-#if defined(BOTAN_HAS_PKCS5_PBE2) && defined(BOTAN_HAS_AES) && defined(BOTAN_HAS_SHA2_32)
+#if defined(BOTAN_HAS_PKCS5_PBES2) && defined(BOTAN_HAS_AES) && defined(BOTAN_HAS_SHA2_32)
 
-         const std::string pbe_algo = "PBE-PKCS5v20(AES-128,SHA-256)";
-         const std::string passphrase = Test::random_password();
-
-         try
-            {
-            Botan::DataSource_Memory data_src(
-               Botan::PKCS8::PEM_encode(key, Test::rng(), passphrase,
-                                        std::chrono::milliseconds(10),
-                                        pbe_algo));
-
-            std::unique_ptr<Botan::Private_Key> loaded(
-               Botan::PKCS8::load_key(data_src, Test::rng(), passphrase));
-
-            result.confirm("recovered private key from encrypted blob", loaded.get() != nullptr);
-            result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
-            try
-               {
-               result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-               }
-            catch(Botan::Lookup_Error&) {}
-            }
-         catch(std::exception& e)
-            {
-            result.test_failure("roundtrip encrypted PEM private key", e.what());
-            }
-
-         try
-            {
-            Botan::DataSource_Memory data_src(
-               Botan::PKCS8::BER_encode(key, Test::rng(), passphrase,
-                                        std::chrono::milliseconds(10),
-                                        pbe_algo));
-
-            std::unique_ptr<Botan::Private_Key> loaded(
-               Botan::PKCS8::load_key(data_src, Test::rng(), passphrase));
-
-            result.confirm("recovered private key from BER blob", loaded.get() != nullptr);
-            result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
-
-            try
-               {
-               result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-               }
-            catch(Botan::Lookup_Error&) {}
-            }
-         catch(std::exception& e)
-            {
-            result.test_failure("roundtrip encrypted BER private key", e.what());
-            }
+         test_pbe_roundtrip(result, key, "PBE-PKCS5v20(AES-128/CBC,SHA-256)", Test::random_password());
 #endif
+
+#if defined(BOTAN_HAS_PKCS5_PBES2) && defined(BOTAN_HAS_AES) && defined(BOTAN_HAS_SCRYPT)
+
+         test_pbe_roundtrip(result, key, "PBES2(AES-128/CBC,Scrypt)", Test::random_password());
+#endif
+
          }
 
       result.end_timer();
