@@ -32,6 +32,8 @@ class Testsuite_RNG final : public Botan::RandomNumberGenerator
          m_a = m_b = m_c = m_d = 0;
          }
 
+      bool accepts_input() const override { return true; }
+
       void add_entropy(const uint8_t data[], size_t len) override
          {
          for(size_t i = 0; i != len; ++i)
@@ -69,7 +71,7 @@ class Testsuite_RNG final : public Botan::RandomNumberGenerator
 
          for(size_t i = 0; i != ROUNDS; ++i)
             {
-            m_a += i;
+            m_a += static_cast<uint32_t>(i);
 
             m_a = Botan::rotl<9>(m_a);
             m_b ^= m_a;
@@ -89,6 +91,7 @@ class Testsuite_RNG final : public Botan::RandomNumberGenerator
 int Test_Runner::run(const Test_Options& opts)
    {
    std::vector<std::string> req = opts.requested_tests();
+   const std::set<std::string> to_skip = opts.skip_tests();
 
    if(req.empty())
       {
@@ -97,10 +100,18 @@ int Test_Runner::run(const Test_Options& opts)
       run the "essentials" to smoke test, then everything else in
       alphabetical order.
       */
-      req = {"block", "stream", "hash", "mac", "modes", "aead",
-             "kdf", "pbkdf", "hmac_drbg", "util"
+
+      std::vector<std::string> default_first = {
+         "block", "stream", "hash", "mac", "modes", "aead",
+         "kdf", "pbkdf", "hmac_drbg", "util"
       };
 
+      for(auto s : default_first)
+         {
+         if(to_skip.count(s) == 0)
+            req.push_back(s);
+         }
+
       std::set<std::string> all_others = Botan_Tests::Test::registered_tests();
 
       if(opts.pkcs11_lib().empty())
@@ -124,6 +135,11 @@ int Test_Runner::run(const Test_Options& opts)
          all_others.erase(f);
          }
 
+      for(const std::string& f : to_skip)
+         {
+         all_others.erase(f);
+         }
+
       req.insert(req.end(), all_others.begin(), all_others.end());
       }
    else if(req.size() == 1 && req.at(0) == "pkcs11")
@@ -145,6 +161,10 @@ int Test_Runner::run(const Test_Options& opts)
       }
 
    output() << "Testing " << Botan::version_string() << "\n";
+
+   const std::string cpuid = Botan::CPUID::to_string();
+   if(cpuid.size() > 0)
+      output() << "CPU flags: " << cpuid << "\n";
    output() << "Starting tests";
 
    if(!opts.pkcs11_lib().empty())
@@ -178,7 +198,7 @@ int Test_Runner::run(const Test_Options& opts)
 
       const size_t failed = run_tests(req, i, opts.test_runs());
       if(failed > 0)
-         return failed;
+         return static_cast<int>(failed);
       }
 
    return 0;
@@ -208,8 +228,7 @@ std::string report_out(const std::vector<Botan_Tests::Test::Result>& results,
 
    for(auto const& result : combined)
       {
-      const bool verbose = false;
-      out << result.second.result_string(verbose);
+      out << result.second.result_string();
       tests_failed += result.second.tests_failed();
       tests_ran += result.second.tests_run();
       }
