@@ -150,7 +150,7 @@ class Test_Tests final : public Test
             {
             Test::Result test_result(testcase_name);
             test_result.test_throws("test_throws", "expected msg",
-                                    []() { throw "not even a std::exception"; });
+                                    []() { throw std::string("not even a std::exception"); });
             verify_failure("test_throws", result, test_result);
             }
 
@@ -188,7 +188,7 @@ class Test_Tests final : public Test
          if(test_result.tests_failed() > 0)
             {
             result.test_success("Got expected failure for " + what);
-            const std::string result_str = test_result.result_string(true);
+            const std::string result_str = test_result.result_string();
 
             result.confirm("result string contains FAIL",
                            result_str.find("FAIL") != std::string::npos);
