@@ -12,7 +12,6 @@
 #include <botan/hex.h>
 #include <botan/parsing.h>
 #include <botan/internal/filesystem.h>
-#include <botan/internal/bit_ops.h>
 #include <botan/internal/stl_util.h>
 
 #if defined(BOTAN_HAS_BIGINT)
@@ -23,6 +22,11 @@
    #include <botan/point_gfp.h>
 #endif
 
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1)
+   #include <stdlib.h>
+   #include <unistd.h>
+#endif
+
 namespace Botan_Tests {
 
 Test::Registration::Registration(const std::string& name, Test* test)
@@ -136,7 +140,7 @@ bool Test::Result::test_throws(const std::string& what, const std::string& expec
 
 bool Test::Result::test_success(const std::string& note)
    {
-   if(Test::log_success())
+   if(Test::options().log_success())
       {
       test_note(note);
       }
@@ -160,7 +164,7 @@ bool Test::Result::test_failure(const std::string& err)
    {
    m_fail_log.push_back(err);
 
-   if(m_who != "Failing Test" && Test::abort_on_first_fail())
+   if(Test::options().abort_on_first_fail() && m_who != "Failing Test")
       {
       std::abort();
       }
@@ -204,21 +208,20 @@ bool Test::Result::test_eq(const char* producer, const std::string& what,
       }
 
    std::vector<uint8_t> xor_diff(std::min(produced_size, expected_size));
-   size_t bits_different = 0;
+   size_t bytes_different = 0;
 
    for(size_t i = 0; i != xor_diff.size(); ++i)
       {
       xor_diff[i] = produced[i] ^ expected[i];
-      bits_different += Botan::hamming_weight(xor_diff[i]);
+      bytes_different += (xor_diff[i] > 0);
       }
 
    err << "\nProduced: " << Botan::hex_encode(produced, produced_size)
        << "\nExpected: " << Botan::hex_encode(expected, expected_size);
 
-   if(bits_different > 0)
+   if(bytes_different > 0)
       {
-      err << "\nXOR Diff: " << Botan::hex_encode(xor_diff)
-          << " (" << bits_different << " bits different)";
+      err << "\nXOR Diff: " << Botan::hex_encode(xor_diff);
       }
 
    return test_failure(err.str());
@@ -311,6 +314,20 @@ bool Test::Result::test_gte(const std::string& what, size_t produced, size_t exp
    return test_success();
    }
 
+bool Test::Result::test_gt(const std::string& what, size_t produced, size_t expected)
+   {
+   if(produced <= expected)
+      {
+      std::ostringstream err;
+      err << m_who;
+      err << " " << what;
+      err << " unexpected result " << produced << " <= " << expected;
+      return test_failure(err.str());
+      }
+
+   return test_success();
+   }
+
 bool Test::Result::test_ne(const std::string& what, const std::string& str1, const std::string& str2)
    {
    if(str1 != str2)
@@ -410,8 +427,10 @@ std::string Test::format_time(uint64_t ns)
    return o.str();
    }
 
-std::string Test::Result::result_string(bool verbose) const
+std::string Test::Result::result_string() const
    {
+   const bool verbose = Test::options().verbose();
+
    if(tests_run() == 0 && !verbose)
       {
       return "";
@@ -495,6 +514,33 @@ Test* Test::get_test(const std::string& test_name)
    return nullptr;
    }
 
+//static
+std::string Test::temp_file_name(const std::string& basename)
+   {
+   // TODO add a --tmp-dir option to the tests to specify where these files go
+
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1)
+
+   // POSIX only calls for 6 'X' chars but OpenBSD allows arbitrary amount
+   std::string mkstemp_basename = "/tmp/" + basename + ".XXXXXXXXXX";
+
+   int fd = ::mkstemp(&mkstemp_basename[0]);
+
+   // error
+   if(fd < 0)
+      {
+      return "";
+      }
+
+   ::close(fd);
+
+   return mkstemp_basename;
+#else
+   // For now just create the temp in the current working directory
+   return basename;
+#endif
+   }
+
 std::string Test::read_data_file(const std::string& path)
    {
    const std::string fsname = Test::data_file(path);
@@ -592,29 +638,37 @@ std::string Test::random_password()
    return Botan::hex_encode(Test::rng().random_vec(len));
    }
 
-Text_Based_Test::Text_Based_Test(const std::string& data_src,
-                                 const std::string& required_keys_str,
-                                 const std::string& optional_keys_str) :
-   m_data_src(data_src)
+std::vector<std::vector<uint8_t>> VarMap::get_req_bin_list(const std::string& key) const
    {
-   if(required_keys_str.empty())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
-      throw Test_Error("Invalid test spec");
+      throw Test_Error("Test missing variable " + key);
       }
 
-   std::vector<std::string> required_keys = Botan::split_on(required_keys_str, ',');
-   std::vector<std::string> optional_keys = Botan::split_on(optional_keys_str, ',');
+   std::vector<std::vector<uint8_t>> bin_list;
 
-   m_required_keys.insert(required_keys.begin(), required_keys.end());
-   m_optional_keys.insert(optional_keys.begin(), optional_keys.end());
-   m_output_key = required_keys.at(required_keys.size() - 1);
+   for(auto&& part : Botan::split_on(i->second, ','))
+      {
+      try
+         {
+         bin_list.push_back(Botan::hex_decode(part));
+         }
+      catch(std::exception& e)
+         {
+         std::ostringstream oss;
+         oss << "Bad input '" << part << "'" << " in binary list key " << key << " - " << e.what();
+         throw Test_Error(oss.str());
+         }
+      }
+
+   return bin_list;
    }
 
-std::vector<uint8_t> Text_Based_Test::get_req_bin(const VarMap& vars,
-      const std::string& key) const
+std::vector<uint8_t> VarMap::get_req_bin(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       throw Test_Error("Test missing variable " + key);
       }
@@ -623,29 +677,28 @@ std::vector<uint8_t> Text_Based_Test::get_req_bin(const VarMap& vars,
       {
       return Botan::hex_decode(i->second);
       }
-   catch(std::exception&)
+   catch(std::exception& e)
       {
-      throw Test_Error("Test invalid hex input '" + i->second + "'" +
-                       + " for key " + key);
+      std::ostringstream oss;
+      oss << "Bad input '" << i->second << "'" << " for key " << key << " - " << e.what();
+      throw Test_Error(oss.str());
       }
    }
 
-std::string Text_Based_Test::get_opt_str(const VarMap& vars,
-      const std::string& key, const std::string& def_value) const
-
+std::string VarMap::get_opt_str(const std::string& key, const std::string& def_value) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       return def_value;
       }
    return i->second;
    }
 
-bool Text_Based_Test::get_req_bool(const VarMap& vars, const std::string& key) const
+bool VarMap::get_req_bool(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       throw Test_Error("Test missing variable " + key);
       }
@@ -664,31 +717,63 @@ bool Text_Based_Test::get_req_bool(const VarMap& vars, const std::string& key) c
       }
    }
 
-size_t Text_Based_Test::get_req_sz(const VarMap& vars, const std::string& key) const
+size_t VarMap::get_req_sz(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       throw Test_Error("Test missing variable " + key);
       }
    return Botan::to_u32bit(i->second);
    }
 
-size_t Text_Based_Test::get_opt_sz(const VarMap& vars, const std::string& key, const size_t def_value) const
+uint8_t VarMap::get_req_u8(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   const size_t s = this->get_req_sz(key);
+   if(s > 256)
+      {
+      throw Test_Error("Invalid " + key + " expected uint8_t got " + std::to_string(s));
+      }
+   return static_cast<uint8_t>(s);
+   }
+
+uint32_t VarMap::get_req_u32(const std::string& key) const
+   {
+   return static_cast<uint32_t>(get_req_sz(key));
+   }
+
+
+size_t VarMap::get_opt_sz(const std::string& key, const size_t def_value) const
+   {
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       return def_value;
       }
    return Botan::to_u32bit(i->second);
    }
 
-std::vector<uint8_t> Text_Based_Test::get_opt_bin(const VarMap& vars,
-      const std::string& key) const
+uint64_t VarMap::get_opt_u64(const std::string& key, const uint64_t def_value) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
+      {
+      return def_value;
+      }
+   try
+      {
+      return std::stoull(i->second);
+      }
+   catch(std::exception&)
+      {
+      throw Test_Error("Invalid u64 value '" + i->second + "'");
+      }
+   }
+
+std::vector<uint8_t> VarMap::get_opt_bin(const std::string& key) const
+   {
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       return std::vector<uint8_t>();
       }
@@ -704,10 +789,10 @@ std::vector<uint8_t> Text_Based_Test::get_opt_bin(const VarMap& vars,
       }
    }
 
-std::string Text_Based_Test::get_req_str(const VarMap& vars, const std::string& key) const
+std::string VarMap::get_req_str(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       throw Test_Error("Test missing variable " + key);
       }
@@ -715,11 +800,10 @@ std::string Text_Based_Test::get_req_str(const VarMap& vars, const std::string&
    }
 
 #if defined(BOTAN_HAS_BIGINT)
-Botan::BigInt Text_Based_Test::get_req_bn(const VarMap& vars,
-      const std::string& key) const
+Botan::BigInt VarMap::get_req_bn(const std::string& key) const
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       throw Test_Error("Test missing variable " + key);
       }
@@ -734,13 +818,12 @@ Botan::BigInt Text_Based_Test::get_req_bn(const VarMap& vars,
       }
    }
 
-Botan::BigInt Text_Based_Test::get_opt_bn(const VarMap& vars,
-      const std::string& key,
-      const Botan::BigInt& def_value) const
+Botan::BigInt VarMap::get_opt_bn(const std::string& key,
+                                 const Botan::BigInt& def_value) const
 
    {
-   auto i = vars.find(key);
-   if(i == vars.end())
+   auto i = m_vars.find(key);
+   if(i == m_vars.end())
       {
       return def_value;
       }
@@ -756,6 +839,24 @@ Botan::BigInt Text_Based_Test::get_opt_bn(const VarMap& vars,
    }
 #endif
 
+Text_Based_Test::Text_Based_Test(const std::string& data_src,
+                                 const std::string& required_keys_str,
+                                 const std::string& optional_keys_str) :
+   m_data_src(data_src)
+   {
+   if(required_keys_str.empty())
+      {
+      throw Test_Error("Invalid test spec");
+      }
+
+   std::vector<std::string> required_keys = Botan::split_on(required_keys_str, ',');
+   std::vector<std::string> optional_keys = Botan::split_on(optional_keys_str, ',');
+
+   m_required_keys.insert(required_keys.begin(), required_keys.end());
+   m_optional_keys.insert(optional_keys.begin(), optional_keys.end());
+   m_output_key = required_keys.at(required_keys.size() - 1);
+   }
+
 std::string Text_Based_Test::get_next_line()
    {
    while(true)
@@ -939,7 +1040,7 @@ std::vector<Test::Result> Text_Based_Test::run()
          results.push_back(Test::Result::Failure(header_or_name,
                                                  test_id + " failed unknown key " + key));
 
-      vars[key] = val;
+      vars.add(key, val);
 
       if(key == m_output_key)
          {
@@ -973,18 +1074,26 @@ std::vector<Test::Result> Text_Based_Test::run()
 
             if(result.tests_failed())
                {
-               if(header.empty())
-                  result.test_note("Test #" + std::to_string(test_cnt) + " failed");
-               else
-                  result.test_note("Test #" + std::to_string(test_cnt) + " " + header + " failed");
+               std::ostringstream oss;
+               oss << "Test # " << test_cnt << " ";
+               if(!header.empty())
+                  oss << header << " ";
+               oss << " failed [Key=" << vars.get_req_str(m_output_key) << "]";
+
+               result.test_note(oss.str());
                }
             results.push_back(result);
             }
          catch(std::exception& e)
             {
-            results.push_back(Test::Result::Failure(header_or_name,
-                                                    "test " + std::to_string(test_cnt) +
-                                                    " failed with exception '" + e.what() + "'"));
+            std::ostringstream oss;
+            oss << "Test # " << test_cnt << " ";
+            if(!header.empty())
+               oss << header << " ";
+            oss << " failed with exception '" << e.what() << "'";
+            oss << " [Key=" << vars.get_req_str(m_output_key) << "]";
+
+            results.push_back(Test::Result::Failure(header_or_name, oss.str()));
             }
 
          if(clear_between_callbacks())
