@@ -119,7 +119,7 @@ std::vector<Test::Result> ECC_Randomized_Tests::run()
 
       Botan::EC_Group group(group_name);
 
-      const Botan::PointGFp& pt = create_random_point(Test::rng(), group);
+      const Botan::PointGFp pt = create_random_point(Test::rng(), group);
       const Botan::BigInt& group_order = group.get_order();
 
       std::vector<Botan::BigInt> blind_ws;
@@ -287,9 +287,19 @@ class EC_Group_Tests : public Test
             const Botan::EC_Group group(group_name);
 
             result.confirm("EC_Group is known", !group.get_curve_oid().empty());
+            result.confirm("EC_Group is considered valid", group.verify_group(Test::rng(), true));
+
             result.test_eq("EC_Group has correct bit size", group.get_p().bits(), group.get_p_bits());
             result.test_eq("EC_Group has byte size", group.get_p().bytes(), group.get_p_bytes());
 
+            result.confirm("Same group is same", group == Botan::EC_Group(group_name));
+
+            const Botan::EC_Group copy(group.get_p(), group.get_a(), group.get_b(),
+                                       group.get_g_x(), group.get_g_y(),
+                                       group.get_order(), group.get_cofactor());
+
+            result.confirm("Same group is same even with copy", group == copy);
+
             const auto pt_mult_by_order = group.get_base_point() * group.get_order();
             result.confirm("Multiplying point by the order results in zero point", pt_mult_by_order.is_zero());
 
@@ -298,6 +308,11 @@ class EC_Group_Tests : public Test
             else
                result.test_ne("Group " + group_name + " A does not equal -3", group.get_a(), group.get_p() - 3);
 
+            if(group.a_is_zero())
+               result.test_eq("Group A is zero", group.get_a(), BigInt(0));
+            else
+               result.test_ne("Group " + group_name + " A does not equal zero", group.get_a(), BigInt(0));
+
             // get a valid point
             Botan::PointGFp p = group.get_base_point() * Test::rng().next_nonzero_byte();
 
@@ -372,7 +387,13 @@ class EC_Group_Tests : public Test
 
       void test_zeropoint(Test::Result& result, const Botan::EC_Group& group)
          {
-         const Botan::PointGFp zero = group.zero_point();
+         Botan::PointGFp zero = group.zero_point();
+
+         result.test_throws("Zero point throws", "Cannot convert zero point to affine",
+                            [&]() { zero.get_affine_x(); });
+         result.test_throws("Zero point throws", "Cannot convert zero point to affine",
+                            [&]() { zero.get_affine_y(); });
+
          const Botan::PointGFp p1 = group.get_base_point() * 2;
 
          result.confirm("point is on the curve", p1.on_the_curve());
@@ -515,6 +536,21 @@ Test::Result test_mult_point()
    return result;
    }
 
+Test::Result test_mixed_points()
+   {
+   Test::Result result("ECC Unit");
+
+   Botan::EC_Group secp256r1("secp256r1");
+   Botan::EC_Group secp384r1("secp384r1");
+
+   const Botan::PointGFp& G256 = secp256r1.get_base_point();
+   const Botan::PointGFp& G384 = secp384r1.get_base_point();
+
+   result.test_throws("Mixing points from different groups",
+                      [&] { Botan::PointGFp p = G256 + G384; });
+   return result;
+   }
+
 Test::Result test_basic_operations()
    {
    Test::Result result("ECC Unit");
@@ -536,10 +572,7 @@ Test::Result test_basic_operations()
    result.test_eq("point addition", simplePlus, exp_simplePlus);
 
    const Botan::PointGFp simpleMinus = p1 - p0;
-   const Botan::PointGFp exp_simpleMinus=  secp160r1.point(Botan::BigInt("425826231723888350446541592701409065913635568770"),
-                                                           Botan::BigInt("203520114162904107873991457957346892027982641970"));
-
-   result.test_eq("point subtraction", simpleMinus, exp_simpleMinus);
+   result.test_eq("point subtraction", simpleMinus, p_G);
 
    const Botan::PointGFp simpleMult = p1 * 123456789;
 
@@ -557,12 +590,9 @@ Test::Result test_enc_dec_compressed_160()
 
    // Test for compressed conversion (02/03) 160bit
    Botan::EC_Group secp160r1("secp160r1");
-
    const std::vector<uint8_t> G_comp = Botan::hex_decode("024A96B5688EF573284664698968C38BB913CBFC82");
-
    const Botan::PointGFp p = secp160r1.OS2ECP(G_comp);
-
-   std::vector<uint8_t> sv_result = p.encode(Botan::PointGFp::COMPRESSED);
+   const std::vector<uint8_t> sv_result = p.encode(Botan::PointGFp::COMPRESSED);
 
    result.test_eq("result", sv_result, G_comp);
    return result;
@@ -673,6 +703,7 @@ class ECC_Unit_Tests final : public Test
          results.push_back(test_point_mult());
          results.push_back(test_point_negative());
          results.push_back(test_mult_point());
+         results.push_back(test_mixed_points());
          results.push_back(test_basic_operations());
          results.push_back(test_enc_dec_compressed_160());
          results.push_back(test_enc_dec_compressed_256());
@@ -703,7 +734,7 @@ class ECC_Invalid_Key_Tests final : public Text_Based_Test
          {
          Test::Result result("ECC invalid keys");
 
-         const std::string encoded = get_req_str(vars, "SubjectPublicKey");
+         const std::string encoded = vars.get_req_str("SubjectPublicKey");
          Botan::DataSource_Memory key_data(Botan::hex_decode(encoded));
 
          try
