@@ -299,12 +299,12 @@ Test::Result test_encoding_options()
    result.test_eq("Hybrid point same size as uncompressed",
                   enc_uncompressed.size(), enc_hybrid.size());
 
-   if(Test::no_avoid_undefined_behavior())
+   if(Test::options().undefined_behavior_allowed())
       {
       auto invalid_format = static_cast<Botan::PointGFp::Compression_Type>(99);
 
       result.test_throws("Invalid point format throws",
-                         "Invalid argument Invalid point encoding for EC_PublicKey",
+                         "Invalid point encoding for EC_PublicKey",
                          [&] { key.set_point_encoding(invalid_format); });
       }
 
